package com.yzdx.AiInterviewer.controller;

import com.yzdx.AiInterviewer.comment.R;
import com.yzdx.AiInterviewer.entity.BackgroundEntity;
import com.yzdx.AiInterviewer.entity.ImagesEntity;
import com.yzdx.AiInterviewer.entity.InvitePromote;
import com.yzdx.AiInterviewer.entity.LogoEntity;
import com.yzdx.AiInterviewer.entity.dto.InterviewRecordDto;
import com.yzdx.AiInterviewer.entity.dto.InvitePromoteDto;
import com.yzdx.AiInterviewer.mapper.InvitePromoteMapper;
import com.yzdx.AiInterviewer.service.*;
import com.yzdx.AiInterviewer.service.VxService.VxInterviewRecordService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/interview")
@Slf4j
public class InterviewController {

    @Autowired
    private InterviewLogoService LogoService;
    @Autowired
    private InterviewImagesService imagesService;
    @Autowired
    private InterviewBackgroundService backgroundService;
    @Autowired
    private InvitePromoteService invitePromoteService;
    @Autowired
    private InterviewNoticeService interviewNoticeService;
    @Autowired
    private VxInterviewRecordService vxInterviewRecordService;
    @Autowired
    private JobPostingService jobPostingService;

    /**
     * 获取公司logo
     * @param encoding 公司编码
     * @return R
     */
    @GetMapping("/get_logoList")
    @ApiOperation(value = "根据编码获取logo图片列表",notes = "")
    public R getLogoList(@ApiParam("传入的值为：(String)encoding") String encoding){

        if(encoding==null){
           return R.error(401,"出错了!请重新登陆");
        }

        List<LogoEntity> logoList = LogoService.getLogoList(encoding);

        return R.success(logoList);

    }

    /**
     * 添加公司Logo
     */

    @ApiImplicitParams({
            @ApiImplicitParam(name = "name",required = true),
            @ApiImplicitParam(name = "encoding",required = true),
            @ApiImplicitParam(name = "image",required = true),
            @ApiImplicitParam(name = "userId",required = true),
            @ApiImplicitParam(name = "filename",required = true),
    })
    @PostMapping("/add_logo")
    @ApiOperation(value = "添加logo图片",notes = "")
    public R addLogo(@RequestBody Map<String,Object> addInfo){
        String name= (String)addInfo.get("name");
        String image= (String)addInfo.get("image");
        String encoding= (String)addInfo.get("encoding");
        Integer userId= (Integer)addInfo.get("userId");
        String filename= (String)addInfo.get("filename");

        if(filename.equals("")){
            return R.error("请添加图片!");
        }
        Integer row = LogoService.addAddLogo(name, encoding, image, userId,filename);
        if(row!=1){
            return R.error("添加失败!");
        }
        return R.success("添加成功!");

    }

    /**
     * 删除公司logo
     * @param id 图片id
     * @return R
     */
    @DeleteMapping("/delete_logo")
    @ApiOperation(value = "删除logo图片",notes = "")
    public R deleteLogoById(@ApiParam("传入的值为：(Integer)id") Integer id){

        Integer rows = LogoService.deleteLogoById(id);
        if(rows!=1){
            return R.error("删除失败，请联系管理员");
        }
        return R.success("删除成功");

    }

    /**
     * 获取公司背景图片2
     * @param encoding 公司编码
     * @return R
     */
    @ApiOperation(value = "根据公司编码搜索背景列表",notes = "")
    @GetMapping("/get_backgroundList")
    public R getBackgroundList(@ApiParam("传入的值：(String)encoding") String encoding){

        if(encoding==null){
            return R.error(401,"出错了!请重新登陆");
        }

        List<BackgroundEntity> backgroundList = backgroundService.getBackgroundList(encoding);

        return R.success(backgroundList);

    }


    /**
     * 添加背景图片
     */

    @ApiImplicitParams({
            @ApiImplicitParam(name = "name",required = true),
            @ApiImplicitParam(name = "encoding",required = true),
            @ApiImplicitParam(name = "image",required = true),
            @ApiImplicitParam(name = "userId",required = true),
            @ApiImplicitParam(name = "filename",required = true),
    })
    @PostMapping("/add_background")
    @ApiOperation(value = "添加背景图片",notes = "")
    public R addBackground(@RequestBody Map<String,Object> addInfo) {
        String name= (String)addInfo.get("name");
        String image= (String)addInfo.get("image");
        String encoding= (String)addInfo.get("encoding");
        Integer userId= (Integer)addInfo.get("userId");
        String filename= (String)addInfo.get("filename");

        if(filename.equals("")){
            return R.error("请添加图片!");
        }
        Integer row = backgroundService.addBackgroundLogo(name, encoding, image, userId,filename);

        if(row!=1){
            return R.error("添加失败!");
        }
        return R.success("添加成功!");

    }

    /**
     * 删除背景图片
     * @param id 公司id
     * @return R
     */
    @DeleteMapping("/delete_background")
    @ApiOperation(value = "删除背景图片",notes = "")
    public R deleteBackgroundById(@ApiParam("传入的值为：(Integer)id") Integer id){

        Integer rows = backgroundService.deleteBackgroundById(id);
        if(rows!=1){
            return R.error("删除失败，请联系管理员");
        }
        return R.success("删除成功");

    }

    /**
     * 获取面试官形象
     * @param encoding 公司编码
     * @return R
     */
    @GetMapping("/get_imagesList")
    @ApiOperation(value = "根据公司编码获取面试官形象图片列表",notes = "")
    public R getImagesList(@ApiParam("传入的值为：(String)encoding") String encoding){

        if(encoding==null){
            return R.error(401,"出错了!请重新登陆");
        }

        List<ImagesEntity> imagesList =imagesService.getImagesList(encoding);

        return R.success(imagesList);

    }

    /**
     * 添加面试官形象照片
     */

    @ApiImplicitParams({
            @ApiImplicitParam(name = "name",required = true),
            @ApiImplicitParam(name = "encoding",required = true),
            @ApiImplicitParam(name = "image",required = true),
            @ApiImplicitParam(name = "userId",required = true),
            @ApiImplicitParam(name = "filename",required = true),
    })
    @PostMapping("/add_image")
    @ApiOperation(value = "添加面试管形象",notes = "")
    public R addImage(@RequestBody Map<String,Object> addInfo) {
        String name= (String)addInfo.get("name");
        String image= (String)addInfo.get("image");
        String encoding= (String)addInfo.get("encoding");
        Integer userId= (Integer)addInfo.get("userId");
        String filename= (String)addInfo.get("filename");
        String video= (String)addInfo.get("video");
        String sex= (String)addInfo.get("sex");

        if(filename.equals("")){
            return R.error("请添加图片!");
        }

        Integer row = imagesService.addImage(name,encoding,image,video,userId,filename,sex);

        if(row!=1){
            return R.error("添加失败!");
        }
        return R.success("添加成功!");

    }

    /**
     * 删除面试官形象
     * @param id 图片id
     * @return R
     */
    @DeleteMapping("/delete_image")
    @ApiOperation(value = "根据形象id删除面试官形象图片",notes = "")
    public R deleteImageById(@ApiParam("传入的值：(Integer)id") Integer id){

        Integer rows = imagesService.deleteImageById(id);
        if(rows!=1){
            return R.error("删除失败，请联系管理员");
        }
        return R.success("删除成功");

    }

    @GetMapping("/getInvitePromote")
    public R getInvitePromoteDto(Integer id){

        InvitePromoteDto invitePromote = invitePromoteService.getInvitePromote(id);

        return R.success(invitePromote);

    }

    @PostMapping("/sendInterviewNotice")
    public R sendInterviewNotice(@RequestBody Map<String,Object> addInfo){
        String encoding=(String) addInfo.get("encoding");
        Integer recipient=(Integer) addInfo.get("inviteUser");
        Integer jobId=(Integer) addInfo.get("jobId");
        Integer postId=(Integer) addInfo.get("postId");
        Integer userId=(Integer) addInfo.get("userId");
        String promote=(String) addInfo.get("promote");
        String selectQuestions=(String) addInfo.get("selectQuestions");
        Integer applicationId=(Integer)addInfo.get("applicationId");
        String endTime=(String)addInfo.get("endTime");

        // 解析原始字符串为 LocalDateTime 对象
        LocalDateTime dateTime = LocalDateTime.parse(endTime, DateTimeFormatter.ISO_DATE_TIME);

        // 格式化 LocalDateTime 对象为所需的日期格式
        String dateString = dateTime.format(DateTimeFormatter.ISO_DATE);


        Integer rows = interviewNoticeService.addInterviewNotice(encoding,recipient,
                jobId, postId,userId,promote,selectQuestions,applicationId,dateString);
        if(rows==-2){
            return R.error("您已经邀请过他面试啦");
        }
        if(rows==-3){
            return R.error("邀请失败,请联系管理员");
        }
        return R.success("邀请成功");

    }
    @GetMapping("/getInterviewRecordList")
    public R getInterviewRecordList(Integer postId,Integer jobId,String encoding,Integer status){

        List<InterviewRecordDto> interviewRecordDtoList=vxInterviewRecordService.getInterviewRecords(postId,jobId,encoding,status);

        return R.success(interviewRecordDtoList);

    }
    @GetMapping("/getInterviewRecordListByUserId")
    public R getInterviewRecordListByUserId(Integer userId){

        List<InterviewRecordDto> interviewRecordDtoList=vxInterviewRecordService.getInterviewRecordListByUserId(userId);

        return R.success(interviewRecordDtoList);

    }

    @GetMapping("/getPostingInfoList")
    public R getPostingInfoList(String encoding){

        List<Map<String,Object>> postingIfoList=jobPostingService.getPostingInfoList(encoding);

        if(postingIfoList==null){
            return R.error("暂未找到招聘岗位信息");
        }

        return R.success(postingIfoList);

    }
    @PostMapping("/agreeUsersPassInterview")
    public R agreeUsersPassInterview(@RequestBody Map<String,Object> updateInfo){

        Integer jobId=(Integer)updateInfo.get("jobId");

        Integer postId=(Integer)updateInfo.get("postId");

        String userList=(String)updateInfo.get("userList");

        Integer userId=(Integer)updateInfo.get("userId");


        Integer rows= vxInterviewRecordService.agreeUsersPassInterview(jobId,postId,userList,userId);

       return R.success("通过成功");



    }

    @PostMapping("/VxSendInterviewNotice")
    public R VxSendInterviewNotice(@RequestBody Map<String,Object> addInfo){
        String encoding=(String) addInfo.get("encoding");
        Integer recipient=(Integer) addInfo.get("inviteUser");
        Integer jobId=(Integer) addInfo.get("jobId");
        Integer postId=(Integer) addInfo.get("postId");
        Integer userId=(Integer) addInfo.get("userId");
        String promote=(String) addInfo.get("promote");
        String selectQuestions=(String) addInfo.get("selectQuestions");
        Integer applicationId=(Integer)addInfo.get("applicationId");
        String endTime=(String)addInfo.get("endTime");

        Integer rows = interviewNoticeService.addInterviewNotice(encoding,recipient,
                jobId, postId,userId,promote,selectQuestions,applicationId,endTime);
        if(rows==-2){
            return R.error("您已经邀请过他面试啦");
        }
        if(rows==-3){
            return R.error("邀请失败,请联系管理员");
        }
        return R.success("邀请成功");

    }
    

}


