package com.yzdx.AiInterviewer.controller.VxController;

import com.yzdx.AiInterviewer.comment.R;
import com.yzdx.AiInterviewer.entity.VxEntityDto.VxCompanyLikeDto;
import com.yzdx.AiInterviewer.entity.VxEntityDto.VxJobLikeDto;
import com.yzdx.AiInterviewer.entity.dto.JobDto;
import com.yzdx.AiInterviewer.service.CompanyService;
import com.yzdx.AiInterviewer.service.JobListService;
import com.yzdx.AiInterviewer.service.VxService.VxCompanyLikeService;
import com.yzdx.AiInterviewer.service.VxService.VxJobLikeService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/vxJob")
public class VxJobController {

    @Autowired
    private JobListService jobListService;

    @Autowired
    private VxJobLikeService vxJobLikeService;

    @Autowired
    private VxCompanyLikeService vxCompanyLikeService;

    @Autowired
    private CompanyService companyService;
//    @GetMapping("/search_job")
//    public R searchJobList(String searchName,String address){
//
//        jobListService.vxSearchJob( searchName, address);
//
//
//    }

    /**
     * 获取岗位关注列表
     * @param userId
     * @return R
     */
    @GetMapping("/getJobLikeList")
    public R getJobLikeList(Integer userId){

       List<VxJobLikeDto> VxJobLikeDtos = vxJobLikeService.getJobLikeListById(userId);

       return R.success(VxJobLikeDtos);

    }

    /**
     * 收藏岗位关注
     *
     */
    @ApiImplicitParams({
            @ApiImplicitParam(name = "jobId",required = true),
            @ApiImplicitParam(name = "userId",required = true),
    })
    @PostMapping("/addJobLike")
    public R addJobLike(@RequestParam(value = "jobId")Integer jobId,
                        @RequestParam(value = "userId")Integer userId){

       Integer rows= vxJobLikeService.addJobLike(jobId,userId);

       if(rows!=1){
           return R.error("收藏失败");
       }
        return R.success("收藏成功");

    }

    /**
     * 取消岗位关注
     * @return
     */
    @DeleteMapping("/cancelJobLike")
    public R cancelJobLike(Integer jobId,Integer userId){
        Integer rows=vxJobLikeService.cancelJobLike(jobId,userId);
        if(rows==-1){
            return R.error("取消收藏失败");
        }
        return R.success("取消收藏成功");
    }

    /**
     * 收藏企业关注
     */
    @ApiImplicitParams({
            @ApiImplicitParam(name = "companyEncoding",required = true),
            @ApiImplicitParam(name = "userId",required = true),
    })
    @PostMapping("/addCompanyLike")
    public R addCompanyLike(@RequestParam(value = "companyEncoding")String companyEncoding,
                            @RequestParam(value = "userId")Integer userId){
        Integer rows=vxCompanyLikeService.addCompanyLike(companyEncoding,userId);

        if(rows!=1){
            return R.error("收藏失败");
        }
        return R.success("收藏成功");
    }

    /**
     * 取消企业关注
     * @param
     * @return
     */
    @DeleteMapping("/cancelCompanyLike")
    public R cancelCompanyLike(String companyEncoding,Integer userId){
        Integer rows=vxCompanyLikeService.cancelCompanyLike(companyEncoding,userId);
        if(rows==-1){
            return R.error("取消关注失败");
        }
        return R.success("取消关注成功");
    }


    /**
     * vx搜索岗位和企业
     * @param searchName
     * @param address
     * @return R
     */
    @GetMapping("/VxSearch")
    public R searchList(String searchName,String address){

        List<VxJobLikeDto> VxJobLikeDtos = jobListService.searchJobList(searchName,address);

        List<VxCompanyLikeDto> VxCompanyLikeDtos = companyService.searchCompanyList(searchName,address);

        Map<String ,Object> result=new HashMap<>();

        result.put("company",VxCompanyLikeDtos);

        result.put("job",VxJobLikeDtos);

        return R.success(result);
    }
    @GetMapping("/getSuggestList")
    public R getSuggestList(Integer userId){

        List<Map<String,Object>> result= jobListService.getSuggestJob(userId);

        return R.success(result);
    }
    /**
     * 获取猜你喜欢
     */
    @GetMapping("/getYourLikeJob")
    public R getYourLikeJob(Integer jobId){
        List<JobDto> list=jobListService.getYourLikeJob(jobId);
        return R.success(list);
    }
}
