package com.yzdx.AiInterviewer.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.yzdx.AiInterviewer.entity.InterviewSetting;
import com.yzdx.AiInterviewer.entity.dto.JobSettingDto;

import java.util.List;
import java.util.Map;

public interface InterviewSettingService extends IService<InterviewSetting> {

    /**
     * 添加岗位设置
     * @param jobId 岗位id
     * @param imagesId 形象id
     * @param backgroundId 面试背景id
     * @param logoId 公司logoId
     * @param Professional 专业题库id
     * @param Comprehensive 综合面试题库id
     * @param Psychology 心理测试id
     * @param userId 创建人id
     * @param jobPromote 岗位promote
     * @param encoding 公司编码
     * @return 影响的行数
     * */

    Integer addJobSetting(Integer jobId,
                          Integer imagesId,
                          Integer backgroundId,
                          Integer  logoId,
                          Integer Professional,
                          Integer Comprehensive,
                          Integer Psychology,
                          Integer userId,
                          String jobPromote,
                          String encoding);
    /**
     * 获取岗位设置列表
     * @param encoding 公司编码
     * @return 返回JobSettingDto列表
     * */
    List<JobSettingDto> getJobSettingList(String encoding);

    /**
     * 搜索面试设置
     * @param jobName 搜索的面试设置的岗位名称
     * @param encoding 公司编码
     * @return 搜索列表
     *
     * */

    List<JobSettingDto> searchJobSetting(String jobName,String encoding);
    /**
     * 根据没你事设置id删除面试设置
     * @param id 岗位设置的id
     * @return 返回影响的行数
     * */
    Integer deleteJobSetting(Integer id);

    /**
     * 修改岗位设置
     * @param id 岗位设置id
     * @param jobId 岗位id
     * @param imagesId 形象id
     * @param backgroundId 面试背景id
     * @param logoId 公司logoId
     * @param Professional 专业题库id
     * @param Comprehensive 综合面试题库id
     * @param Psychology 心理测试id
     * @param userId 创建人id
     * @param jobPromote 岗位promote
     * @param encoding 公司编码
     * @return 影响的行数
     * */

    Integer updateJobSetting(Integer id,
                            Integer jobId,
                          Integer imagesId,
                          Integer backgroundId,
                          Integer  logoId,
                          Integer Professional,
                          Integer Comprehensive,
                          Integer Psychology,
                          Integer userId,
                          String jobPromote,
                          String encoding);

    /**
     * 根据jobid获取面试信息
     * @param jobId 岗位id
     * @return 面试设置信息
     *
     * */

    Map<String, Object> getJobSettingByJobId(Integer jobId);

}
