package com.yzdx.AiInterviewer.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.yzdx.AiInterviewer.entity.JobEntity;
import com.yzdx.AiInterviewer.entity.VxEntityDto.VxJobDetailDto;
import com.yzdx.AiInterviewer.entity.dto.JobDto;
import com.yzdx.AiInterviewer.entity.VxEntityDto.VxJobLikeDto;

import java.util.List;
import java.util.Map;

public interface JobListService extends IService<JobEntity> {
/**
 * 添加岗位
 * @param userId 添加人的id
 * @param encoding 添加人所在的公司ID
 * @param details 岗位详情
 * @param address 工作地址
 * @param address_detail 工作详细地址
 * @param endTime 下班时间
 * @param startTime 上班时间
 * @param jobName 岗位名称
 * @param jobTips 岗位标签
 * @param jobType 岗位类型
 * @param max_number 招聘最高人数
 * @param min_number 招聘最少人数
 * @param max_salary 招聘最高工资
 * @param min_salary 招聘最低工资
 * @param requirement 学历要求
 * @return 影响行数
 *
 *
 * */

    Integer addJob(List<String> address,
                   String address_detail,
                   String details,
                   String jobName,
                   String startTime,
                   String endTime,
                   List<String> jobTips,
                   Integer jobType,
                   String min_salary,
                   String max_salary,
                   String min_number,
                   String max_number,
                   Integer requirement,
                   Integer userId,
                   String encoding);

    /**
     * @param encoding 公司编码
     * @return 岗位列表
     * */
    List<JobDto> getJobList(String encoding);
    /**
     * 搜索岗位
     * @param jobName 搜索的岗位名称
     * @param jobType 搜索的岗位类型
     * @param encoding 公司编码
     * @return 返回jonList
     *
     * */
    List<JobDto> searchJob(String jobName,String jobType,String encoding);

    /**
     * 根据id查岗位
     * @param id 岗位id
     * @return 岗位对象
     * */
    VxJobDetailDto getJobById(Integer id);
    /**
     * 修改岗位
     * @param userId 添加人的id
     * @param encoding 添加人所在的公司ID
     * @param details 岗位详情
     * @param address 工作地址
     * @param address_detail 工作详细地址
     * @param endTime 下班时间
     * @param startTime 上班时间
     * @param jobName 岗位名称
     * @param jobTips 岗位标签
     * @param jobType 岗位类型
     * @param max_number 招聘最高人数
     * @param min_number 招聘最少人数
     * @param max_salary 招聘最高工资
     * @param min_salary 招聘最低工资
     * @param requirement 学历要求
     * @return 影响行数
     * */
    Integer updateJob(
                    Integer id,
                    List<String> address,
                   String address_detail,
                   String details,
                   String jobName,
                   String startTime,
                   String endTime,
                   List<String> jobTips,
                   Integer jobType,
                   String min_salary,
                   String max_salary,
                   String min_number,
                   String max_number,
                   Integer requirement,
                   Integer userId,
                   String encoding);
    /**
     * 禁用/启用岗位状态
     * @param id 岗位id
     * @param userId 更改人id
     * @return 影响的行数
     *
     * */
    Integer changeJobStatus(Integer id, Integer userId);
    /**
     * 删除岗位
     * @param id 岗位id
     * @return 影响的行数
     *
     * */
    Integer deleteJob(Integer id);



    //------------------------------------------------vx-------------------------------------------
/**
 * vx搜索岗位
 * @param searchName 搜索名称
 * @param address 所在地址
 * @return jobDto列表
 *
 * */

    List<VxJobLikeDto> searchJobList(String searchName, String address);


    /**
     * 根据ID进行岗位推荐
     * @param userId 用户ID
     * @return jobList
     *
     * */
    List<Map<String,Object>> getSuggestJob(Integer userId);

    /**
     * 根据岗位id查猜你喜欢
     * @param jobId
     * @return
     */
    List<JobDto> getYourLikeJob(Integer jobId);
}
