package com.yzdx.AiInterviewer.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.yzdx.AiInterviewer.entity.SharedQuestionBank;
import com.yzdx.AiInterviewer.entity.dto.SharedQuestionBankDto;

import java.util.List;

public interface SharedQuestionBankService extends IService<SharedQuestionBank> {

    /**
     * 添加分享题库
     * @param typeName 题库名称
     * @param type 题库类型
     * @param description 题库描述
     * @param userId 创建人
     * @return 创建的SharedQuestionBank对象
     * */
    SharedQuestionBank addSharedQuestionBank(String typeName,Integer type,String description,Integer userId,String encoding);
    /**
     * 获取分享题库列表
     * @return 分享题库列表
     * */
    List<SharedQuestionBankDto> getSharedQuestionBank();
    /**
     * 根据id查找分享题库对象
     * @param bankId 题库ID
     * @return 题库对象
     *
     * */

    SharedQuestionBank getSharedQuestionBankById(Integer bankId);
    /**
     * 根据公司编码查找分享题库
     * @param  encoding 公司编码
     * @return 分享题库
     */
     SharedQuestionBank getShareQuestionTypeByCompany(String encoding);

    /**
     * 根据题库名称搜索分享题库
     * @param searchName 题库名称
     * @return 分享题库列表
     */
     List<SharedQuestionBankDto> searchSharedQuestionType(String searchName);

    /**
     * 根据分享题库添加题库
     * @param SharedBankIds
     * @param userId
     * @param encoding
     * @return 题库
     */

     String addQuestionTypeFromShare(List<Integer>SharedBankIds,Integer userId,String encoding);
    /**
     * 根据公司编码获取本公司分享的题库
     * @param encoding
     * @return 题库列表
     */
    List<SharedQuestionBankDto> getOurQuestionTypeList(String encoding);

    /**
     * 根据公司编码和题库名称搜索题库
     * @param encoding
     * @param searchName
     * @return 题库列表
     */
    List<SharedQuestionBankDto> searchOurSharedQuestionType(String encoding,String searchName);

    /**
     * 根据分享题库id修改题库信息
     * @param id 分享题库id
     * @param typeName 题库名称
     * @param description 题库描述
     * @param type 题库类型
     * @param userId 更该人id
     * @return 修改的行数
     * */
    Integer editOurSharedQuestionType(Integer id,String typeName,String description,Integer type,Integer userId);

    /**
     * 根据分享题库id删除分享题库
     * @param id 分享题库id
     * @return 影响的行数
     * */
    Integer deleteOurSharedQuestionType(Integer id);



}
