package com.yzdx.AiInterviewer.service;


import com.baomidou.mybatisplus.extension.service.IService;
import com.yzdx.AiInterviewer.comment.R;
import com.yzdx.AiInterviewer.entity.User;

import java.util.List;

public interface UserService extends IService<User> {

    /** 管理员登录业务
     * @param phone 输入的账号
     * @param encoding 输入的公司编码
     * @param password 输入的密码
     * @return boolean,ture登陆成功,false：登陆失败
     *
     */

    R adminLogin(String phone, String encoding, String password);

    /**
     * 根据用户id查询用户信息
     * @param userid 前端穿过来的用户id
     * @return 根据id查询到的user对象
     * */
    User getUserById(Integer userid);
    /**
     * 管理员登出
     * @param token 用户登录时返回的token
     * @return 1成功，0失败
     *
     * */
    Integer adminLogout(String token);
    /**
     * 获取管理员列表
     * @param encoding 公司编码
     * @return 管理员列表
     *
     * */
    List<User> getAdminList(String encoding);
/**
 * 搜索管理员
 * @param encoding 公司编码
 * @param name 搜索管理员的名字
 * @return 搜索管理员信息
 * */
    List<User> searchAdmin(String name,String encoding);
    /**
     * 添加管理员
     * @param encoding 公司编码
     * @param userId 操作人的id
     * @param username 添加人的姓名
     * @param phone 添加人的手机号/找好
     * @param role 添加的人的权限
     * @return 影响的行数
     *
     * */
    Integer addAdmin(String encoding,Integer userId,String username,String phone,String role);
    /**
     * 更新管理员信息
     * @param userId 操作人的id
     * @param deleteId 删除人的id
     * @return 影响的行数
     *
     * */

    Integer deleteAdmin(Integer deleteId,Integer userId);
    /**
     * 更换头像
     * @param userId 更改人的id
     * @param avatar 更改人头像的地址
     * @return User信息
     * */
    User changeAvatar(Integer userId,String avatar);

    /**
     * 更改密码
     * @param userId 更改人的id
     * @param password 新密码
     * @param oldPassword 旧密码
     *
     * @return  user对象
     *
     * */
    R resetPassword(Integer userId,String oldPassword,String password);

    /**
     * @param userId 操作人的id
     * @param updateId 更新人的id
     * @param username 修改的名称
     * @param age 修改的年龄
     * @param email 修改的电子邮箱
     * @param sex 修改的性别
     * @return 返回user对象
     *
     * */
    User updateUserInfo(Integer userId,Integer updateId,String username,String age,String email,String sex);


    R vxLogin(String phone);

    /**
     * 管理员修改属下密码
     * @param userId 操作人id
     * @param password 修改的密码
     * @param updateUserId 被更新人的id
     * */

    Integer adminEditPassword(Integer userId, Integer updateUserId, String password);
}

