package com.yzdx.AiInterviewer.service.VxService.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yzdx.AiInterviewer.entity.*;
import com.yzdx.AiInterviewer.entity.VxEntityDto.VxInterviewApplicationDto;
import com.yzdx.AiInterviewer.entity.vxEntity.VxInterviewApplication;
import com.yzdx.AiInterviewer.mapper.ApplicationCountMapper;
import com.yzdx.AiInterviewer.mapper.JobMapper;
import com.yzdx.AiInterviewer.mapper.ResumeMapper;
import com.yzdx.AiInterviewer.mapper.UserMapper;
import com.yzdx.AiInterviewer.mapper.VxMapper.VxInterviewApplicationMapper;
import com.yzdx.AiInterviewer.service.VxService.VxInterviewApplicationService;
import com.yzdx.AiInterviewer.utiles.TimeUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.Period;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class VxInterviewApplicationServiceImpl extends ServiceImpl<VxInterviewApplicationMapper, VxInterviewApplication> implements VxInterviewApplicationService {
    @Autowired
    private VxInterviewApplicationMapper vxInterviewApplicationMapper;
    @Autowired
    private ResumeMapper resumeMapper;
    @Autowired
    private JobMapper jobMapper;
    @Autowired
    private UserMapper userMapper;
    @Autowired
    private ApplicationCountMapper applicationCountMapper;

    @Override
    public Integer addInterviewApplication(Integer jobId, Integer userId, Integer postingId, String companyEncoding) {
        LambdaQueryWrapper<VxInterviewApplication> queryWrapper=new LambdaQueryWrapper<>();
        queryWrapper.eq(VxInterviewApplication::getJobId,jobId).eq(VxInterviewApplication::getPostingId,postingId)
                .eq(VxInterviewApplication::getUserId,userId).eq(BaseEntity::getCreateTime,TimeUtil.getTime());
        VxInterviewApplication vxInterviewApplication = vxInterviewApplicationMapper.selectOne(queryWrapper);
        if(vxInterviewApplication!=null){
          vxInterviewApplication.setCreateTime(TimeUtil.getTime());
          vxInterviewApplicationMapper.updateById(vxInterviewApplication);
        }
        VxInterviewApplication vxInterviewApplication1=new VxInterviewApplication();
        vxInterviewApplication1.setJobId(jobId);
        vxInterviewApplication1.setCompanyEncoding(companyEncoding);
        vxInterviewApplication1.setPostingId(postingId);
        vxInterviewApplication1.setUserId(userId);
        //获取用户简历id
        LambdaQueryWrapper<Resume> queryWrapper1=new LambdaQueryWrapper<>();

        queryWrapper1.eq(Resume::getUserId,userId);

        Resume resume = resumeMapper.selectOne(queryWrapper1);
        vxInterviewApplication1.setResumeId(resume.getId());
        vxInterviewApplication1.setCreateTime(TimeUtil.getTime());
        vxInterviewApplication1.setUpdateTime(TimeUtil.getTime());
        vxInterviewApplication1.setCreateUser(userId);
        vxInterviewApplication1.setUpdateUser(userId);
        Integer insert = vxInterviewApplicationMapper.insert(vxInterviewApplication1);

        LambdaQueryWrapper<ApplicationCount> queryWrapper2=new LambdaQueryWrapper<>();

        queryWrapper2.eq(ApplicationCount::getEncoding,companyEncoding).eq(ApplicationCount::getCreateTime,TimeUtil.getTime());

        ApplicationCount applicationCount = applicationCountMapper.selectOne(queryWrapper2);

        applicationCount.setCount(applicationCount.getCount()+1);

        applicationCountMapper.updateById(applicationCount);


        return insert;
    }
    @Override
    public List<VxInterviewApplicationDto> getApplicationList(String companyEncoding, Integer jobId) {

        LambdaQueryWrapper<VxInterviewApplication> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.eq(VxInterviewApplication::getCompanyEncoding,companyEncoding).eq(VxInterviewApplication::getJobId,jobId);

        List<VxInterviewApplication> VxInterviewApplicationList = vxInterviewApplicationMapper.selectList(queryWrapper);

        List<VxInterviewApplicationDto>VxInterviewApplicationDtoList =VxInterviewApplicationList.stream().map(item->{

            VxInterviewApplicationDto newVxInterviewApplicationDto=new VxInterviewApplicationDto();

            BeanUtils.copyProperties(item,newVxInterviewApplicationDto);

            JobEntity jobEntity = jobMapper.selectById(item.getJobId());

            newVxInterviewApplicationDto.setJobName(jobEntity.getJobName());

            User user = userMapper.selectById(item.getUserId());

            newVxInterviewApplicationDto.setUserName(user.getUsername());

            Resume resume = resumeMapper.selectById(item.getResumeId());

            //解析在校经历，计算出学历

            JSONArray educationJSONArray= JSON.parseArray(resume.getEducationBackground());

            long difference =0;

            String eduBack="";

            String school="";

            String major="";
            for (int i = 0; i < educationJSONArray.size(); i++) {

                Map<String,Object> educationMap= (Map<String,Object>)educationJSONArray.get(i);

                String lastEndTime= (String)educationMap.get("endTime");

                // 将字符串转换为日期对象
                Date date = new Date(Integer.valueOf(lastEndTime) - 1900, 0, 1); // 注意：年份需减去1900

                // 获取日期对象的时间戳（毫秒数）

                long timestamp = date.getTime();

                // 获取当前时间的时间戳
                long currentTimestamp = System.currentTimeMillis();

                // 计算时间戳差值
                long differenceTime = currentTimestamp - timestamp;

                //初始化值
                if(i==0){
                    difference=differenceTime;
                    eduBack=(String)educationMap.get("eduDegree");
                    school=(String)educationMap.get("school");
                    major=(String)educationMap.get("major");

                    continue;
                }
                //判断最近的毕业年份
                if(differenceTime<difference){
                    difference=differenceTime;
                    eduBack=(String)educationMap.get("eduDegree");
                    school=(String)educationMap.get("school");
                    major=(String)educationMap.get("major");

                    continue;
                }

            }

            resume.setEducation(eduBack);

            Integer rows= resumeMapper.updateById(resume);

            Map<String,Object> result=new HashMap<>();

            if(rows!=1){
                return null;
            }
            int experience=0;
            //计算工作时间
            JSONArray jobJSONArray=JSON.parseArray(resume.getWorkExperience());

            if(jobJSONArray.size()==0) {
                //放入工作时间
                result.put("experience", experience);
            }else{
                for (int i = 0; i < jobJSONArray.size(); i++) {

                    Map<String,Object> jobJSON=(Map<String, Object>) jobJSONArray.get(i);

                    // 将字符串日期转换为LocalDate对象
                    LocalDate startDate = LocalDate.parse(jobJSON.get("startTime")+ "-01");
                    LocalDate endDate = LocalDate.parse(jobJSON.get("endTime") + "-01");

                    // 计算年份差异
                    Period period = Period.between(startDate, endDate);
                    int yearDifference = period.getYears();
                    experience+=yearDifference;
                }
                //放入工作时间
                result.put("experience", experience);
            }
            result.put("userName",resume.getName());
            result.put("userImgUrl",resume.getAvatar());
            result.put("eduBack",eduBack);
            result.put("gender",resume.getSex());
            result.put("school",school);
            result.put("major",major);
            result.put("phone",resume.getPhone());
            result.put("resume",resume);

            newVxInterviewApplicationDto.setResume(result);

            newVxInterviewApplicationDto.setCreateTime(item.getCreateTime());

            return newVxInterviewApplicationDto;

        }).collect(Collectors.toList());

        return VxInterviewApplicationDtoList;
    }

    @Override
    public Integer disagreeApplication(Integer id) {
        VxInterviewApplication vxInterviewApplication = vxInterviewApplicationMapper.selectById(id);

        vxInterviewApplication.setStatus(3);

       return vxInterviewApplicationMapper.updateById(vxInterviewApplication);

    }

    @Override
    public List<VxInterviewApplicationDto> searchApplicationByName(String searchName, String encoding) {

        LambdaQueryWrapper<User> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.like(User::getUsername,searchName);

        List<User> users = userMapper.selectList(queryWrapper);

        List<VxInterviewApplication> vxInterviewApplicationList=new ArrayList<>();



        for (User user:users) {
            LambdaQueryWrapper<VxInterviewApplication> queryWrapper1=new LambdaQueryWrapper<>();

            queryWrapper1.eq(VxInterviewApplication::getCompanyEncoding,encoding).eq(VxInterviewApplication::getUserId,user.getId());

            List<VxInterviewApplication> vxInterviewApplications = vxInterviewApplicationMapper.selectList(queryWrapper1);

            vxInterviewApplicationList.addAll(vxInterviewApplications);
        }
        
        List<VxInterviewApplicationDto>VxInterviewApplicationDtoList =vxInterviewApplicationList.stream().map(item->{

            VxInterviewApplicationDto newVxInterviewApplicationDto=new VxInterviewApplicationDto();

            BeanUtils.copyProperties(item,newVxInterviewApplicationDto);

            JobEntity jobEntity = jobMapper.selectById(item.getJobId());

            newVxInterviewApplicationDto.setJobName(jobEntity.getJobName());

            User user = userMapper.selectById(item.getUserId());

            newVxInterviewApplicationDto.setUserName(user.getUsername());

            Resume resume = resumeMapper.selectById(item.getResumeId());

            //解析在校经历，计算出学历

            JSONArray educationJSONArray= JSON.parseArray(resume.getEducationBackground());

            long difference =0;

            String eduBack="";

            String school="";

            String major="";
            for (int i = 0; i < educationJSONArray.size(); i++) {

                Map<String,Object> educationMap= (Map<String,Object>)educationJSONArray.get(i);

                String lastEndTime= (String)educationMap.get("endTime");

                // 将字符串转换为日期对象
                Date date = new Date(Integer.valueOf(lastEndTime) - 1900, 0, 1); // 注意：年份需减去1900

                // 获取日期对象的时间戳（毫秒数）

                long timestamp = date.getTime();

                // 获取当前时间的时间戳
                long currentTimestamp = System.currentTimeMillis();

                // 计算时间戳差值
                long differenceTime = currentTimestamp - timestamp;

                //初始化值
                if(i==0){
                    difference=differenceTime;
                    eduBack=(String)educationMap.get("eduDegree");
                    school=(String)educationMap.get("school");
                    major=(String)educationMap.get("major");

                    continue;
                }
                //判断最近的毕业年份
                if(differenceTime<difference){
                    difference=differenceTime;
                    eduBack=(String)educationMap.get("eduDegree");
                    school=(String)educationMap.get("school");
                    major=(String)educationMap.get("major");

                    continue;
                }

            }

            resume.setEducation(eduBack);

            Integer rows= resumeMapper.updateById(resume);

            Map<String,Object> result=new HashMap<>();

            if(rows!=1){
                return null;
            }
            int experience=0;
            //计算工作时间
            JSONArray jobJSONArray=JSON.parseArray(resume.getWorkExperience());

            if(jobJSONArray.size()==0) {
                //放入工作时间
                result.put("experience", experience);
            }else{
                for (int i = 0; i < jobJSONArray.size(); i++) {

                    Map<String,Object> jobJSON=(Map<String, Object>) jobJSONArray.get(i);

                    // 将字符串日期转换为LocalDate对象
                    LocalDate startDate = LocalDate.parse(jobJSON.get("startTime")+ "-01");
                    LocalDate endDate = LocalDate.parse(jobJSON.get("endTime") + "-01");

                    // 计算年份差异
                    Period period = Period.between(startDate, endDate);
                    int yearDifference = period.getYears();
                    experience+=yearDifference;
                }
                //放入工作时间
                result.put("experience", experience);
            }
            result.put("userName",resume.getName());
            result.put("userImgUrl",resume.getAvatar());
            result.put("eduBack",eduBack);
            result.put("gender",resume.getSex());
            result.put("school",school);
            result.put("major",major);
            result.put("phone",resume.getPhone());
            result.put("resume",resume);

            newVxInterviewApplicationDto.setResume(result);

            return newVxInterviewApplicationDto;

        }).collect(Collectors.toList());

        return VxInterviewApplicationDtoList;



    }
}
