package com.yzdx.AiInterviewer.service.VxService.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yzdx.AiInterviewer.entity.*;
import com.yzdx.AiInterviewer.entity.VxEntityDto.VxInterviewRecordDto;
import com.yzdx.AiInterviewer.entity.dto.InterviewRecordDto;
import com.yzdx.AiInterviewer.entity.vxEntity.VxInterviewNotice;
import com.yzdx.AiInterviewer.entity.vxEntity.VxInterviewRecord;
import com.yzdx.AiInterviewer.mapper.*;
import com.yzdx.AiInterviewer.mapper.VxMapper.VxInterviewNoticeMapper;
import com.yzdx.AiInterviewer.mapper.VxMapper.VxInterviewRecordMapper;
import com.yzdx.AiInterviewer.service.CompanyService;
import com.yzdx.AiInterviewer.service.VxService.VxInterviewRecordService;
import com.yzdx.AiInterviewer.utiles.EmailUtil;
import com.yzdx.AiInterviewer.utiles.TimeUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
@Service
public class VxInterviewRecordServiceImpl extends ServiceImpl<VxInterviewRecordMapper, VxInterviewRecord> implements VxInterviewRecordService {
    @Autowired
    private VxInterviewRecordMapper vxInterviewRecordMapper;
    @Autowired
    private VxInterviewNoticeMapper vxInterviewNoticeMapper;
    @Autowired
    private JobMapper jobMapper;
    @Autowired
    private CompanyMapper companyMapper;

    @Autowired
    private CompanyService companyService;

    @Autowired
    private JobPostingMapper jobPostingMapper;

    @Autowired
    private TalentMapper talentMapper;
    @Autowired
    private UserMapper userMapper;

    @Override
    public List<VxInterviewRecordDto> getInterviewRecordList(Integer userId) {
            LambdaQueryWrapper<VxInterviewRecord> queryWrapper=new LambdaQueryWrapper<>();
            queryWrapper.eq(VxInterviewRecord::getInterviewer,userId);
            List<VxInterviewRecord> vxInterviewRecords = vxInterviewRecordMapper.selectList(queryWrapper);
            List<VxInterviewRecordDto> vxInterviewRecordDtos=vxInterviewRecords.stream().map(item->{
                VxInterviewRecordDto vxInterviewRecordDto=new VxInterviewRecordDto();

                BeanUtils.copyProperties(item,vxInterviewRecordDto);

                Company companyDetail = companyService.getCompanyDetail(item.getCompanyEncoding());

                vxInterviewRecordDto.setCompanyName(companyDetail.getCompanyName());

                vxInterviewRecordDto.setCompanyLogoUrl(companyDetail.getCompanyLogo());

                vxInterviewRecordDto.setInterviewTime(item.getCreateTime());

                VxInterviewNotice vxInterviewNotice = vxInterviewNoticeMapper.selectById(item.getNoticeId());

                JobEntity jobEntity = jobMapper.selectById(vxInterviewNotice.getJobId());

                vxInterviewRecordDto.setJob(jobEntity.getJobName());


                return vxInterviewRecordDto;
            }).collect(Collectors.toList());
        return vxInterviewRecordDtos;
    }

    @Override
    public List<VxInterviewRecordDto> searchInterviewRecordList(String searchCompany,Integer userId) {

        LambdaQueryWrapper<Company> queryWrapper1=new LambdaQueryWrapper<>();

        queryWrapper1.like(Company::getCompanyName,searchCompany);

        List<Company> companies = companyMapper.selectList(queryWrapper1);

        if(companies.size()==0){

            return null;
        }

        List<VxInterviewRecordDto> vxInterviewRecordDtoList=new ArrayList<>();
        for (Company company:companies) {

            LambdaQueryWrapper<VxInterviewRecord> queryWrapper=new LambdaQueryWrapper<>();

            queryWrapper.eq(VxInterviewRecord::getInterviewer,userId).eq(VxInterviewRecord::getCompanyEncoding,company.getEncoding());

            List<VxInterviewRecord> vxInterviewRecords = vxInterviewRecordMapper.selectList(queryWrapper);

            List<VxInterviewRecordDto> vxInterviewRecordDtos=vxInterviewRecords.stream().map(item->{

                VxInterviewRecordDto vxInterviewRecordDto=new VxInterviewRecordDto();

                BeanUtils.copyProperties(item,vxInterviewRecordDto);

                Company companyDetail = companyService.getCompanyDetail(item.getCompanyEncoding());

                vxInterviewRecordDto.setCompanyName(companyDetail.getCompanyName());

                vxInterviewRecordDto.setCompanyLogoUrl(companyDetail.getCompanyLogo());

                vxInterviewRecordDto.setInterviewTime(item.getCreateTime());

                VxInterviewNotice vxInterviewNotice = vxInterviewNoticeMapper.selectById(item.getNoticeId());

                JobEntity jobEntity = jobMapper.selectById(vxInterviewNotice.getJobId());

                vxInterviewRecordDto.setJob(jobEntity.getJobName());

                return vxInterviewRecordDto;

            }).collect(Collectors.toList());

            vxInterviewRecordDtoList.addAll(vxInterviewRecordDtos);
        }


        return vxInterviewRecordDtoList;
    }

    @Override
    public Integer addInterviewRecord(String encoding, String comments, Integer interviewer, Integer noticeId,Integer talentId,String detail) {

        LambdaQueryWrapper<VxInterviewRecord> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.eq(VxInterviewRecord::getInterviewer,interviewer).eq(VxInterviewRecord::getNoticeId,noticeId);

        VxInterviewRecord vxInterviewRecord = vxInterviewRecordMapper.selectOne(queryWrapper);

        if(vxInterviewRecord!=null){
            return -1;
        }
        VxInterviewRecord vxInterviewRecord1=new VxInterviewRecord();

        vxInterviewRecord1.setCompanyEncoding(encoding);

        vxInterviewRecord1.setComments(comments);

        vxInterviewRecord1.setInterviewer(interviewer);

        vxInterviewRecord1.setNoticeId(noticeId);

        vxInterviewRecord1.setCreateUser(interviewer);

        vxInterviewRecord1.setCreateTime(TimeUtil.getTime());

        vxInterviewRecord1.setDetail(detail);


       return vxInterviewRecordMapper.insert(vxInterviewRecord1);

    }

    @Override
    public List<InterviewRecordDto> getInterviewRecords(Integer postId, Integer jobId, String encoding,Integer status) {


        LambdaQueryWrapper<VxInterviewRecord> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.eq(VxInterviewRecord::getCompanyEncoding,encoding).eq(VxInterviewRecord::getStatus,status);

        List<VxInterviewRecord> vxInterviewRecords = vxInterviewRecordMapper.selectList(queryWrapper);

        vxInterviewRecords= vxInterviewRecords.stream().map(item->{

            VxInterviewNotice vxInterviewNotice = vxInterviewNoticeMapper.selectById(item.getNoticeId());

            if(vxInterviewNotice.getJobId()==jobId&&vxInterviewNotice.getPostId()==postId){
                return item;
            }

            return null;

        }).collect(Collectors.toList());

        if(vxInterviewRecords.size()==0||(vxInterviewRecords.size()==1&&vxInterviewRecords.get(0)==null)){
            return null;
        }
        List<InterviewRecordDto> interviewRecordDtoList=vxInterviewRecords.stream().map(item->{

            InterviewRecordDto interviewRecordDto=new InterviewRecordDto();

            BeanUtils.copyProperties(item,interviewRecordDto);

            Map<String,Object> postInfo=new HashMap<>();

            VxInterviewRecord vxInterviewRecord = vxInterviewRecordMapper.selectById(item.getId());

            VxInterviewNotice vxInterviewNotice = vxInterviewNoticeMapper.selectById(vxInterviewRecord.getNoticeId());

            postInfo.put("postId",vxInterviewNotice.getPostId());

            postInfo.put("jobId",vxInterviewNotice.getJobId());

            interviewRecordDto.setPostInfo(postInfo);

            Map<String,Object> talentInfo=new HashMap<>();

            Talent talent = talentMapper.selectById(item.getTalentId());

            talentInfo.put("talentInfo",talent);

            interviewRecordDto.setPostInfo(talentInfo);

            User userInfo=userMapper.selectById(item.getInterviewer());

            userInfo.setPassword(null);

            userInfo.setSalt(null);

            interviewRecordDto.setUserInfo(userInfo);

            return interviewRecordDto;

        }).collect(Collectors.toList());

        return interviewRecordDtoList;



    }

    @Override
    public Integer agreeUsersPassInterview(Integer jobId, Integer postId, String userList,Integer Id) {
        JSONArray userIDS=JSON.parseArray(userList);

        for (int i = 0; i <userIDS.size() ; i++) {
            Integer userId =(Integer) userIDS.get(i);

            LambdaQueryWrapper<VxInterviewNotice> queryWrapper=new LambdaQueryWrapper<>();

            queryWrapper.eq(VxInterviewNotice::getJobId,jobId).eq(VxInterviewNotice::getPostId,postId)
                    .eq(VxInterviewNotice::getRecipient,userId);

            VxInterviewNotice vxInterviewNotice = vxInterviewNoticeMapper.selectOne(queryWrapper);

            LambdaQueryWrapper<VxInterviewRecord> queryWrapper1=new LambdaQueryWrapper<>();

            queryWrapper1.eq(VxInterviewRecord::getInterviewer,userId).eq(VxInterviewRecord::getNoticeId,vxInterviewNotice.getId());

            VxInterviewRecord vxInterviewRecord = vxInterviewRecordMapper.selectOne(queryWrapper1);

            vxInterviewRecord.setStatus(2);

            vxInterviewRecord.setUpdateUser(Id);

            vxInterviewRecord.setUpdateTime(TimeUtil.getTime());

            User user = userMapper.selectById(vxInterviewRecord.getInterviewer());

            LambdaQueryWrapper<Company> queryWrapper2=new LambdaQueryWrapper<>();

            queryWrapper2.eq(Company::getEncoding,vxInterviewRecord.getCompanyEncoding());

            Company company = companyMapper.selectOne(queryWrapper2);

            try {
//            "<h1>"+userName+"先生,你好!</h1> 您已获得本公司"+"["+jobName+"]AI面试资格，请打开微信小程序，点击面试通知查看详情"
                //"扬城直聘："+companyName
                String detail= "<h1>"+user.getUsername()+(user.getSex().equals("男")?"先生":"女士")+"你好!</h1> 感谢您参加我们的面试，我们很高兴地通知您，您已经通过了面试，成为我们团队的一员。\n" +
                        "\n" +
                        "您在面试中展示了出色的专业技能和沟通能力，我们对您的表现印象深刻。我们相信您能够为我们的项目做出重要的贡献，并与我们的同事们建立良好的合作关系。\n" +
                        "\n" +
                        "您的入职日期是2024年1月1日，您的工作地点是江苏省扬州市邗江区扬子江中路760号。您的岗位是java开发工程师，您的月薪是10000元。您的直属上司是李经理，他将在您入职后为您安排工作和培训。\n" +
                        "\n" +
                        "请在收到本邮件后，尽快回复确认您是否接受我们的录用，并告知您的联系方式。如果您有任何问题或疑虑，请随时与我们联系。\n" +
                        "\n" +
                        "我们期待着您的加入，祝您一切顺利！\n";
                String title="扬城直聘："+company.getCompanyName();
                EmailUtil.sendEmail(user.getEmail(),title,detail);
            } catch (Exception e) {
                e.printStackTrace();
                return -3;
            }

            vxInterviewRecordMapper.updateById(vxInterviewRecord);

        }
        return 1;
    }

    @Override
    public List<InterviewRecordDto> getInterviewRecordListByUserId(Integer userId) {
       LambdaQueryWrapper<VxInterviewRecord> queryWrapper=new LambdaQueryWrapper<>();

       queryWrapper.eq(VxInterviewRecord::getInterviewer,userId);

        List<VxInterviewRecord> vxInterviewRecords = vxInterviewRecordMapper.selectList(queryWrapper);

        if(vxInterviewRecords.size()==0||(vxInterviewRecords.size()==1&&vxInterviewRecords.get(0)==null)){
            return null;
        }
        List<InterviewRecordDto> interviewRecordDtoList=vxInterviewRecords.stream().map(item->{

            InterviewRecordDto interviewRecordDto=new InterviewRecordDto();

            BeanUtils.copyProperties(item,interviewRecordDto);

            Map<String,Object> postInfo=new HashMap<>();

            VxInterviewRecord vxInterviewRecord = vxInterviewRecordMapper.selectById(item.getId());

            VxInterviewNotice vxInterviewNotice = vxInterviewNoticeMapper.selectById(vxInterviewRecord.getNoticeId());

            postInfo.put("postId",vxInterviewNotice.getPostId());

            postInfo.put("jobId",vxInterviewNotice.getJobId());

            interviewRecordDto.setPostInfo(postInfo);

            Map<String,Object> talentInfo=new HashMap<>();

            Talent talent = talentMapper.selectById(item.getTalentId());

            talentInfo.put("talentInfo",talent);

            interviewRecordDto.setPostInfo(talentInfo);

            User userInfo=userMapper.selectById(item.getInterviewer());

            userInfo.setPassword(null);

            userInfo.setSalt(null);

            interviewRecordDto.setUserInfo(userInfo);

            return interviewRecordDto;

        }).collect(Collectors.toList());

        return interviewRecordDtoList;



    }
}
