package com.yzdx.AiInterviewer.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yzdx.AiInterviewer.entity.Company;
import com.yzdx.AiInterviewer.entity.JobEntity;
import com.yzdx.AiInterviewer.entity.VxEntityDto.VxCompanyDetailDto;
import com.yzdx.AiInterviewer.entity.VxEntityDto.VxCompanyLikeDto;
import com.yzdx.AiInterviewer.entity.dto.JobDto;
import com.yzdx.AiInterviewer.entity.vxEntity.VxCompanyLike;
import com.yzdx.AiInterviewer.mapper.CompanyMapper;
import com.yzdx.AiInterviewer.mapper.JobMapper;
import com.yzdx.AiInterviewer.mapper.VxMapper.VxCompanyLikeMapper;
import com.yzdx.AiInterviewer.mapper.VxMapper.VxJobLikeMapper;
import com.yzdx.AiInterviewer.service.CompanyService;
import com.yzdx.AiInterviewer.service.JobListService;
import com.yzdx.AiInterviewer.utiles.TimeUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class CompanyServiceImpl extends ServiceImpl<CompanyMapper, Company> implements CompanyService {

    @Autowired
    private CompanyMapper companyMapper;

    @Autowired
    private JobMapper jobMapper;
    @Autowired
    private VxCompanyLikeMapper vxCompanyLikeMapper;

    @Override
    public VxCompanyDetailDto getCompanyDetail(String encoding) {

        LambdaQueryWrapper<Company> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.eq(Company::getEncoding,encoding);

        Company company = companyMapper.selectOne(queryWrapper);

        VxCompanyDetailDto vxCompanyDetailDto=new VxCompanyDetailDto();

        BeanUtils.copyProperties(company,vxCompanyDetailDto);

        LambdaQueryWrapper<VxCompanyLike> queryWrapper1=new LambdaQueryWrapper<>();

        queryWrapper1.eq(VxCompanyLike::getCompanyEncoding,company.getEncoding());

        VxCompanyLike vxCompanyLike = vxCompanyLikeMapper.selectOne(queryWrapper1);

        if(vxCompanyLike!=null){
            vxCompanyDetailDto.setIsLike(1);
            return vxCompanyDetailDto;
        }
            vxCompanyDetailDto.setIsLike(0);
        return vxCompanyDetailDto;
    }

    @Override
    public Integer updateCompanyDetail(String address, String addressDetail, String type,String companyAllName,String companyDetail, String companyName,
                                       String creditCode, String encoding, String established, String images, String legalRepresentative,
                                       String treatment, String registeredCapital, Integer userId,String companyLogo,
                                       String staffNumber) {
        if (address==null||addressDetail==null||companyDetail==null||companyName==null||creditCode==null||established==null||images==null||legalRepresentative==null
                ||treatment==null||registeredCapital==null||userId==null||staffNumber==null||companyLogo==null||type==null||companyAllName==null){
            return -1;
        }
        //通过公司编码查询公司
        LambdaQueryWrapper<Company> queryWrapper=new LambdaQueryWrapper<>();
        queryWrapper.eq(Company::getEncoding,encoding);
        Company company = companyMapper.selectOne(queryWrapper);
        company.setAddress(address);
        company.setAddressDetail(addressDetail);
        company.setCompanyDetail(companyDetail);
        company.setCompanyName(companyName);
        company.setCreditCode(creditCode);
        company.setEstablished(established);
        company.setImages(images);
        company.setType(type);
        company.setCompanyAllName(companyAllName);
        company.setLegalRepresentative(legalRepresentative);
        company.setTreatment(treatment);
        company.setRegisteredCapital(registeredCapital);
        company.setUpdateTime(TimeUtil.getTime());
        company.setUpdateUser(userId);
        company.setCompanyLogo(companyLogo);
        company.setStaffNumber(staffNumber);
        Integer i = companyMapper.updateById(company);
        return i;

    }
    //-------------------------------------------------vx----------------------------------------------

    @Override
    public List<VxCompanyLikeDto> searchCompanyList(String searchName, String address) {

        LambdaQueryWrapper<Company> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.like(Company::getCompanyName,searchName).like(Company::getAddress,address);

        List<Company> companies = companyMapper.selectList(queryWrapper);

        List<VxCompanyLikeDto> CompanyLikeDtos=companies.stream().map(item->{

            VxCompanyLikeDto vxCompanyLikeDto=new VxCompanyLikeDto();

            vxCompanyLikeDto.setCompanyId(item.getId());

            vxCompanyLikeDto.setCompanyName(item.getCompanyName());
            //得到公司类型


            vxCompanyLikeDto.setCompanyType(item.getType());
            //得到公司logo url

            vxCompanyLikeDto.setCompanyImgUrl(item.getCompanyLogo());
            //得到企业人数

            vxCompanyLikeDto.setStaffNum(item.getStaffNumber());
            //得到相关信息
            LambdaQueryWrapper<JobEntity > queryWrapper1=new LambdaQueryWrapper<>();

            queryWrapper1.eq(JobEntity::getCompanyEncoding,item.getEncoding());

            List<JobEntity> jobList = jobMapper.selectList(queryWrapper1);

            List<Map<String,Object>> jobRelatives=jobList.stream().map(i->{
                Map<String,Object> jobRelative=new HashMap<>();

                jobRelative.put("jobname",i.getJobName());
                jobRelative.put("salary",i.getMinSalary()+"-"+i.getMaxSalary());
                return jobRelative;

            }).collect(Collectors.toList());
            vxCompanyLikeDto.setJobRelative(jobRelatives);
            return vxCompanyLikeDto;
        }).collect(Collectors.toList());
        return CompanyLikeDtos;
    }


}
