package com.yzdx.AiInterviewer.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yzdx.AiInterviewer.entity.Company;
import com.yzdx.AiInterviewer.entity.InvitePromote;
import com.yzdx.AiInterviewer.entity.JobEntity;
import com.yzdx.AiInterviewer.entity.User;
import com.yzdx.AiInterviewer.entity.VxEntityDto.VxNoticeDto;
import com.yzdx.AiInterviewer.entity.vxEntity.VxInterviewApplication;
import com.yzdx.AiInterviewer.entity.vxEntity.VxInterviewNotice;
import com.yzdx.AiInterviewer.mapper.InvitePromoteMapper;
import com.yzdx.AiInterviewer.mapper.JobMapper;
import com.yzdx.AiInterviewer.mapper.UserMapper;
import com.yzdx.AiInterviewer.mapper.VxMapper.VxInterviewApplicationMapper;
import com.yzdx.AiInterviewer.mapper.VxMapper.VxInterviewNoticeMapper;
import com.yzdx.AiInterviewer.service.CompanyService;
import com.yzdx.AiInterviewer.service.InterviewNoticeService;
import com.yzdx.AiInterviewer.utiles.EmailUtil;
import com.yzdx.AiInterviewer.utiles.TimeUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

@Service
public class InterviewNoticeServiceImpl extends ServiceImpl<VxInterviewNoticeMapper, VxInterviewNotice> implements InterviewNoticeService {

    @Autowired
    private VxInterviewNoticeMapper vxInterviewNoticeMapper;
    @Autowired
    private UserMapper userMapper;
    @Autowired
    private CompanyService companyService;

    @Autowired
    private JobMapper jobMapper;
    @Autowired
    private InvitePromoteMapper invitePromoteMapper;
    @Autowired
    private VxInterviewApplicationMapper vxInterviewApplicationMapper;

    @Override
    public Integer addInterviewNotice(String encoding, Integer recipient, Integer jobId, Integer postId,
                                     Integer userId,String promote,String selectQuestions,Integer applicationId,String endTime) {

        LambdaQueryWrapper<VxInterviewNotice> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.eq(VxInterviewNotice::getCompanyEncoding,encoding).eq(VxInterviewNotice::getJobId,jobId)
                .eq(VxInterviewNotice::getPostId,postId);

        VxInterviewNotice vxInterviewNotice = vxInterviewNoticeMapper.selectOne(queryWrapper);

        if(vxInterviewNotice!=null){

            return -2;
        }

        VxInterviewNotice newVxInterviewNotice=new VxInterviewNotice();

        newVxInterviewNotice.setCompanyEncoding(encoding);

        newVxInterviewNotice.setRecipient(recipient);

        newVxInterviewNotice.setJobId(jobId);

        newVxInterviewNotice.setInterviewEndTime(endTime);

        newVxInterviewNotice.setPostId(postId);


        LambdaQueryWrapper<InvitePromote> queryWrapper1=new LambdaQueryWrapper<>();

        queryWrapper1.eq(InvitePromote::getJobId,jobId).eq(InvitePromote::getPostId,postId).eq(InvitePromote::getInviteUser,recipient);

        InvitePromote invitePromote1 = invitePromoteMapper.selectOne(queryWrapper1);

        if(invitePromote1==null){
            //添加promote表
            InvitePromote invitePromote=new InvitePromote();

            invitePromote.setEncoding(encoding);

            invitePromote.setJobId(jobId);

            invitePromote.setPostId(postId);

            invitePromote.setQuestion(selectQuestions);

            invitePromote.setPromote(promote);

            invitePromote.setInviteUser(recipient);

            invitePromote.setCreateUser(userId);

            invitePromote.setCreateTime(TimeUtil.getTime());

            invitePromoteMapper.insert(invitePromote);

            LambdaQueryWrapper<InvitePromote> queryWrapper2=new LambdaQueryWrapper<>();

            queryWrapper2.eq(InvitePromote::getJobId,jobId).eq(InvitePromote::getPostId,postId).eq(InvitePromote::getInviteUser,recipient);

            invitePromote1=invitePromoteMapper.selectOne(queryWrapper2);

        }else{
            invitePromote1.setEncoding(encoding);

            invitePromote1.setJobId(jobId);

            invitePromote1.setPostId(postId);

            invitePromote1.setQuestion(selectQuestions);

            invitePromote1.setPromote(promote);

            invitePromote1.setInviteUser(recipient);

            invitePromote1.setUpdateUser(userId);

            invitePromote1.setUpdateTime(TimeUtil.getTime());

            invitePromoteMapper.updateById(invitePromote1);

        }


        newVxInterviewNotice.setInviteId(invitePromote1.getId());

        newVxInterviewNotice.setCreateTime(TimeUtil.getTime());

        newVxInterviewNotice.setUpdateTime(TimeUtil.getTime());

        newVxInterviewNotice.setCreateUser(userId);

        newVxInterviewNotice.setUpdateUser(userId);

        //查询用户的信息
        User user = userMapper.selectById(recipient);

        //查询公司信息
        Company companyDetail = companyService.getCompanyDetail(encoding);

        //查询岗位信息
        JobEntity jobEntity = jobMapper.selectById(jobId);


        String companyName=companyDetail.getCompanyName();

        String email=user.getEmail();

        String userName=user.getUsername();

        String jobName=jobEntity.getJobName();
        if(email==null||userName==null||companyName==null||jobName==null){
            return -3;
        }
        try {
//            "<h1>"+userName+"先生,你好!</h1> 您已获得本公司"+"["+jobName+"]AI面试资格，请打开微信小程序，点击面试通知查看详情"
            //"扬城直聘："+companyName
            String detail= "<h1>"+userName+(user.getSex().equals("男")?"先生":"女士")+"你好!</h1> 您已获得本公司"+"["+jobName+"]AI面试资格，请打开微信小程序，点击面试通知查看详情";
            String title="扬城直聘："+companyName;
            EmailUtil.sendEmail(email,title,detail);
        } catch (Exception e) {
            e.printStackTrace();
            return -3;
        }

        VxInterviewApplication vxInterviewApplication = vxInterviewApplicationMapper.selectById(applicationId);

        vxInterviewApplication.setStatus(2);

        vxInterviewApplicationMapper.updateById(vxInterviewApplication);

        Integer rows = vxInterviewNoticeMapper.insert(newVxInterviewNotice);


        return rows;
    }

    @Override
    public List<VxNoticeDto> getInterviewNoticeByUserId(Integer userId) {

        LambdaQueryWrapper<VxInterviewNotice> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.eq(VxInterviewNotice::getRecipient,userId);

        List<VxInterviewNotice> vxInterviewNotices = vxInterviewNoticeMapper.selectList(queryWrapper);

        List<VxNoticeDto> VxNoticeDtoList=vxInterviewNotices.stream().map(item->{

            VxNoticeDto vxNoticeDto=new VxNoticeDto();

            BeanUtils.copyProperties(item,vxNoticeDto);

            vxNoticeDto.setCompanyEncoding(item.getCompanyEncoding());

            Company companyDetail = companyService.getCompanyDetail(item.getCompanyEncoding());

            vxNoticeDto.setCompanyName(companyDetail.getCompanyName());

            vxNoticeDto.setInterviewSendTime(item.getCreateTime());

            vxNoticeDto.setCompanyLogoUrl(companyDetail.getCompanyLogo());

            User user = userMapper.selectById(item.getRecipient());

            vxNoticeDto.setInterviewer(user.getUsername());

            JobEntity jobEntity = jobMapper.selectById(item.getJobId());

            vxNoticeDto.setJob(jobEntity.getJobName());

            vxNoticeDto.setInterviewEndTime(item.getInterviewEndTime());

            return vxNoticeDto;

        }).collect(Collectors.toList());

        return VxNoticeDtoList;
    }

    @Override
    public String getInterviewPromote(Integer noticeId) {
        VxInterviewNotice vxInterviewNotice = vxInterviewNoticeMapper.selectById(noticeId);
        if(vxInterviewNotice==null){
            return null;
        }
        InvitePromote invitePromote = invitePromoteMapper.selectById(vxInterviewNotice.getInviteId());

        if(invitePromote==null){
            return null;
        }
        return invitePromote.getPromote();
    }


}
