package com.yzdx.AiInterviewer.service.impl;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yzdx.AiInterviewer.entity.Company;
import com.yzdx.AiInterviewer.entity.JobEntity;
import com.yzdx.AiInterviewer.entity.Resume;
import com.yzdx.AiInterviewer.entity.VxEntityDto.VxJobDetailDto;
import com.yzdx.AiInterviewer.entity.dto.JobDto;
import com.yzdx.AiInterviewer.entity.VxEntityDto.VxJobLikeDto;
import com.yzdx.AiInterviewer.entity.vxEntity.VxJobLike;
import com.yzdx.AiInterviewer.mapper.JobMapper;
import com.yzdx.AiInterviewer.mapper.VxMapper.VxJobLikeMapper;
import com.yzdx.AiInterviewer.service.CompanyService;
import com.yzdx.AiInterviewer.service.JobListService;
import com.yzdx.AiInterviewer.service.ResumeService;
import com.yzdx.AiInterviewer.utiles.MyStringUtils;
import com.yzdx.AiInterviewer.utiles.TimeUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class JobListServiceImpl extends ServiceImpl<JobMapper, JobEntity> implements JobListService {

    @Autowired
    private JobMapper jobMapper;
    @Autowired
    private CompanyService companyService;
    @Autowired
    private ResumeService resumeService;
    @Autowired
    private VxJobLikeMapper vxJobLikeMapper;
    @Override
    public Integer addJob(List<String> address, String address_detail, String details, String jobName, String startTime, String endTime, List<String> jobTips, Integer jobType, String min_salary, String max_salary, String min_number, String max_number, Integer requirement, Integer userId, String encoding) {
        //查看岗位名称是否存在
        LambdaQueryWrapper<JobEntity> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.eq(JobEntity::getJobName,jobName);

        JobEntity existsJob = jobMapper.selectOne(queryWrapper);

        if(existsJob!=null){
            //存在，不能添加
            return -2;
        }

//        不存在，可以添加
        JobEntity newJob=new JobEntity();
        newJob.setAddress(address.toString());
        newJob.setAddressDetail(address_detail);
        newJob.setJobName(jobName);
        newJob.setCompanyEncoding(encoding);
        newJob.setJobTips(jobTips.toString());
        newJob.setJobType(jobType);
        newJob.setDetails(details);
        newJob.setEndTime(endTime);
        newJob.setStartTime(startTime);
        newJob.setMaxNumber(max_number);
        newJob.setRequirement(requirement);
        newJob.setMinNumber(min_number);
        newJob.setMaxSalary(max_salary);
        newJob.setMinSalary(min_salary);
        newJob.setCreateUser(userId);
        newJob.setCreateTime(TimeUtil.getTime());

      return jobMapper.insert(newJob);

    }

    @Override
    public List<JobDto> getJobList(String encoding) {
        LambdaQueryWrapper<JobEntity> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.eq(JobEntity::getCompanyEncoding,encoding);

        List<JobEntity> jobEntities = jobMapper.selectList(queryWrapper);
        List<JobDto> jobDtoList = jobEntities.stream().map((item) -> {

        JobDto jobDto=new JobDto();

        jobDto.setId(item.getId());
        jobDto.setJobName(item.getJobName());
        jobDto.setJobType(item.getJobType());
        jobDto.setDetail(item.getDetails());
        String number=item.getMinNumber()+"-"+item.getMaxNumber();
        jobDto.setNumber(number);
        String address= MyStringUtils.convertString(item.getAddress())+item.getAddressDetail();
        jobDto.setAddress(address);
        String salary=item.getMinSalary()+"-"+item.getMaxSalary();
        jobDto.setSalary(salary);
        jobDto.setStatus(item.getStatus());
        jobDto.setCreateUser(item.getCreateUser());
        jobDto.setCreateTime(item.getCreateTime());
        jobDto.setUpdateUser(item.getUpdateUser());
        jobDto.setUpdateTime(item.getUpdateTime());
        jobDto.setJobTips(item.getJobTips());
        return jobDto;
        }).collect(Collectors.toList());

        return jobDtoList;

    }

    @Override
    public List<JobDto> searchJob(String jobName, String jobType, String encoding) {
        if(!jobName.equals("")){
            if(!jobType.equals("")){
                LambdaQueryWrapper<JobEntity> queryWrapper =new LambdaQueryWrapper<>();
                queryWrapper.like(JobEntity::getJobName,jobName).eq(JobEntity::getJobType,jobType).eq(JobEntity::getCompanyEncoding,encoding);

                List<JobEntity> jobEntities = jobMapper.selectList(queryWrapper);
                List<JobDto> jobDtoList = jobEntities.stream().map((item) -> {

                    JobDto jobDto=new JobDto();

                    jobDto.setId(item.getId());
                    jobDto.setJobName(item.getJobName());
                    jobDto.setJobType(item.getJobType());
                    jobDto.setDetail(item.getDetails());
                    String number=item.getMinNumber()+"-"+item.getMaxNumber();
                    jobDto.setNumber(number);
                    String address= MyStringUtils.convertString(item.getAddress())+item.getAddressDetail();
                    jobDto.setAddress(address);
                    String salary=item.getMinSalary()+"-"+item.getMaxSalary();
                    jobDto.setSalary(salary);
                    jobDto.setStatus(item.getStatus());
                    jobDto.setCreateUser(item.getCreateUser());
                    jobDto.setCreateTime(item.getCreateTime());
                    jobDto.setUpdateUser(item.getUpdateUser());
                    jobDto.setUpdateTime(item.getUpdateTime());
                    return jobDto;
                }).collect(Collectors.toList());
                return jobDtoList;
            }else{
             LambdaQueryWrapper<JobEntity> queryWrapper=new LambdaQueryWrapper<>();

             queryWrapper.like(JobEntity::getJobName,jobName).eq(JobEntity::getCompanyEncoding,encoding);

                List<JobEntity> jobEntities = jobMapper.selectList(queryWrapper);
                List<JobDto> jobDtoList = jobEntities.stream().map((item) -> {

                    JobDto jobDto=new JobDto();

                    jobDto.setId(item.getId());
                    jobDto.setJobName(item.getJobName());
                    jobDto.setJobType(item.getJobType());
                    jobDto.setDetail(item.getDetails());
                    String number=item.getMinNumber()+"-"+item.getMaxNumber();
                    jobDto.setNumber(number);
                    String address= MyStringUtils.convertString(item.getAddress())+item.getAddressDetail();
                    jobDto.setAddress(address);
                    String salary=item.getMinSalary()+"-"+item.getMaxSalary();
                    jobDto.setSalary(salary);
                    jobDto.setStatus(item.getStatus());
                    jobDto.setCreateUser(item.getCreateUser());
                    jobDto.setCreateTime(item.getCreateTime());
                    jobDto.setUpdateUser(item.getUpdateUser());
                    jobDto.setUpdateTime(item.getUpdateTime());
                    return jobDto;
                }).collect(Collectors.toList());
                return jobDtoList;
            }
        }else{
            if(!jobType.equals("")){
                LambdaQueryWrapper<JobEntity> queryWrapper=new LambdaQueryWrapper<>();

                queryWrapper.eq(JobEntity::getJobType,jobType).eq(JobEntity::getCompanyEncoding,encoding);

                List<JobEntity> jobEntities = jobMapper.selectList(queryWrapper);
                List<JobDto> jobDtoList = jobEntities.stream().map((item) -> {

                    JobDto jobDto=new JobDto();

                    jobDto.setId(item.getId());
                    jobDto.setJobName(item.getJobName());
                    jobDto.setJobType(item.getJobType());
                    jobDto.setDetail(item.getDetails());
                    String number=item.getMinNumber()+"-"+item.getMaxNumber();
                    jobDto.setNumber(number);
                    String address= MyStringUtils.convertString(item.getAddress())+item.getAddressDetail();
                    jobDto.setAddress(address);
                    String salary=item.getMinSalary()+"-"+item.getMaxSalary();
                    jobDto.setSalary(salary);
                    jobDto.setStatus(item.getStatus());
                    jobDto.setCreateUser(item.getCreateUser());
                    jobDto.setCreateTime(item.getCreateTime());
                    jobDto.setUpdateUser(item.getUpdateUser());
                    jobDto.setUpdateTime(item.getUpdateTime());
                    return jobDto;
                }).collect(Collectors.toList());
                return jobDtoList;
            }else{
                return null;
            }
        }
    }

    @Override
    public VxJobDetailDto getJobById(Integer id) {
        LambdaQueryWrapper<JobEntity> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.eq(JobEntity::getId,id);

        JobEntity jobEntity = jobMapper.selectOne(queryWrapper);

        VxJobDetailDto vxJobDetailDto=new VxJobDetailDto();

        BeanUtils.copyProperties(jobEntity,vxJobDetailDto);

        LambdaQueryWrapper<VxJobLike> queryWrapper1=new LambdaQueryWrapper<>();

        queryWrapper1.eq(VxJobLike::getJobId,jobEntity.getId());

        VxJobLike vxJobLike = vxJobLikeMapper.selectOne(queryWrapper1);

        if(vxJobLike!=null){
            vxJobDetailDto.setIsLike(1);
            return vxJobDetailDto;
        }
        vxJobDetailDto.setIsLike(0);
        return vxJobDetailDto;

    }

    @Override
    public Integer updateJob(Integer id, List<String> address, String address_detail, String details, String jobName, String startTime, String endTime, List<String> jobTips, Integer jobType, String min_salary, String max_salary, String min_number, String max_number, Integer requirement, Integer userId, String encoding) {
        //验证岗位是否存在
        LambdaQueryWrapper<JobEntity> jobEntityLambdaQueryWrapper=new LambdaQueryWrapper<>();

        jobEntityLambdaQueryWrapper.eq(JobEntity::getId,id);

        JobEntity findJob = jobMapper.selectOne(jobEntityLambdaQueryWrapper);
        if(findJob==null&&!findJob.getCompanyEncoding().equals(encoding)){
            return -1;
        }
        //查询修改的岗位名是否重名
        LambdaQueryWrapper<JobEntity> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.eq(JobEntity::getJobName,jobName);

        JobEntity existJobNamEntity=jobMapper.selectOne(queryWrapper);

        if(existJobNamEntity!=null&&!(existJobNamEntity.getId()==id))
        {
//            重名
            return -2;
        }
        //存在则进行修改
        findJob.setAddress(address.toString());
        findJob.setAddressDetail(address_detail);
        findJob.setJobName(jobName);
        findJob.setJobTips(jobTips.toString());
        findJob.setJobType(jobType);
        findJob.setDetails(details);
        findJob.setEndTime(endTime);
        findJob.setStartTime(startTime);
        findJob.setMaxNumber(max_number);
        findJob.setRequirement(requirement);
        findJob.setMinNumber(min_number);
        findJob.setMaxSalary(max_salary);
        findJob.setMinSalary(min_salary);
        findJob.setUpdateTime(TimeUtil.getTime());
        findJob.setUpdateUser(userId);
       return jobMapper.updateById(findJob);

    }

    @Override
    public Integer changeJobStatus(Integer id, Integer userId) {
        //查询岗位是否存在
        LambdaQueryWrapper<JobEntity> queryWrapper=new LambdaQueryWrapper<>();
        
        queryWrapper.eq(JobEntity::getId,id);

        JobEntity findJob = jobMapper.selectOne(queryWrapper);
        //不存在，返回-1
        if(findJob==null){
            return -1;
        }
        Integer status = findJob.getStatus();
        if(status==0){
            findJob.setStatus(1);
        }else {
            findJob.setStatus(0);
        }
        findJob.setUpdateUser(userId);
        findJob.setUpdateTime(TimeUtil.getTime());
         return jobMapper.updateById(findJob);
    }

    @Override
    public Integer deleteJob(Integer id) {
        LambdaQueryWrapper<JobEntity> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.eq(JobEntity::getId,id);

        JobEntity findJob = jobMapper.selectOne(queryWrapper);

        if(findJob==null){
            return -1;
        }
        return jobMapper.delete(queryWrapper);
    }


    //----------------------------------------------------vx----------------------------------------------
    @Override
    public List<VxJobLikeDto> searchJobList(String searchName, String address) {
        LambdaQueryWrapper<JobEntity> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.like(JobEntity::getJobName,searchName).like(JobEntity::getAddress,address);

        List<JobEntity> jobEntities = jobMapper.selectList(queryWrapper);

        List<VxJobLikeDto> vxJobLikeDtoList=jobEntities.stream().map(item->{

            VxJobLikeDto vxJobLikeDto=new VxJobLikeDto();

            BeanUtils.copyProperties(item,vxJobLikeDto);

            vxJobLikeDto.setJobName(item.getJobName());

            vxJobLikeDto.setSalary(item.getMinSalary()+"-"+item.getMaxSalary());

            Company companyDetail = companyService.getCompanyDetail(item.getCompanyEncoding());

            vxJobLikeDto.setCompany(companyDetail.getCompanyName());

            vxJobLikeDto.setStaffNumber(companyDetail.getStaffNumber());

            vxJobLikeDto.setLabel(item.getJobTips());

            vxJobLikeDto.setPosition(item.getAddressDetail());

            return vxJobLikeDto;

        }).collect(Collectors.toList());

        return vxJobLikeDtoList;

    }

    ///需要改----------------------------------------------------------------
    @Override
    public List<Map<String,Object>> getSuggestJob(Integer userId) {
        List<Map<String,Object>> result=new ArrayList<>();
        //先获取用户的岗位期望
        Map<String,Object> resultResume = resumeService.getResume(userId);
        List<String> suggestJobs=new ArrayList<>();
        JSONArray jobExpectationJSONs =null;
        if(resultResume==null||userId==null){
            LambdaQueryWrapper<JobEntity> addQueryWrapper =new LambdaQueryWrapper<>();

            addQueryWrapper.last("limit 5");

            List<JobEntity> suggestJobEntities = jobMapper.selectList(addQueryWrapper);

            suggestJobs=suggestJobEntities.stream().map(item->{

                String jobName=item.getJobName();

                return jobName ;

            }).collect(Collectors.toList());
        }

        Resume resume=(Resume) resultResume.get("resume");
        String jobExpectation = resume.getJobExpectation();

       jobExpectationJSONs = (JSONArray) JSON.parse(jobExpectation);



        if(jobExpectationJSONs.size()<=5){
            LambdaQueryWrapper<JobEntity> addQueryWrapper =new LambdaQueryWrapper<>();

            addQueryWrapper.last("limit 5");

            List<JobEntity> suggestJobEntities = jobMapper.selectList(addQueryWrapper);

            suggestJobs=suggestJobEntities.stream().map(item->{

                String jobName=item.getJobName();

                return jobName ;

            }).collect(Collectors.toList());
        }



        for (int i = 0; i < jobExpectationJSONs.size(); i++) {

            String jobExpectationJSON = jobExpectationJSONs.getString(i);

            JSONObject jsonObject=(JSONObject)JSON.parse(jobExpectationJSON);

            suggestJobs.add(jsonObject.getString("jobName"));
        }


        for (int i = 0; i < suggestJobs.size(); i++) {

            String jobName = suggestJobs.get(i);

            LambdaQueryWrapper<JobEntity> queryWrapper=new LambdaQueryWrapper<>();

            try {
                queryWrapper.like(JobEntity::getJobName,jobName).or().like(JobEntity::getJobName,jobName.toString().substring(0,2));
            } catch (Exception e) {
                continue;
            }

            List<JobEntity> jobEntities1 = jobMapper.selectList(queryWrapper);

            if(jobEntities1==null){

                continue;
            }

            List<VxJobLikeDto> vxJobLikeDtoList=jobEntities1.stream().map(item->{

                VxJobLikeDto vxJobLikeDto=new VxJobLikeDto();

                BeanUtils.copyProperties(item,vxJobLikeDto);

                vxJobLikeDto.setJobName(item.getJobName());

                vxJobLikeDto.setSalary(item.getMinSalary()+"-"+item.getMaxSalary());

                Company companyDetail = companyService.getCompanyDetail(item.getCompanyEncoding());

                vxJobLikeDto.setCompany(companyDetail.getCompanyName());

                vxJobLikeDto.setStaffNumber(companyDetail.getStaffNumber());

                vxJobLikeDto.setLabel(item.getJobTips());

                vxJobLikeDto.setPosition(item.getAddressDetail());

                return vxJobLikeDto;

            }).collect(Collectors.toList());

            Map<String,Object> jobEntitys=new HashMap<>();
            jobEntitys.put("suggestJobName",jobName);
            jobEntitys.put("suggestJobList",vxJobLikeDtoList);

            result.add(jobEntitys);

        }
        return result;
    }
    @Override
    public List<JobDto> getYourLikeJob(Integer jobId) {
        LambdaQueryWrapper<JobEntity> queryWrapper=new LambdaQueryWrapper<>();
        queryWrapper.like(JobEntity::getId,jobId);
        List<JobEntity> vxJobLikes = jobMapper.selectList(queryWrapper);
        List<JobDto> jobDtos=vxJobLikes.stream().map(item->{
            JobDto vxJobLikeDto=new JobDto();
            BeanUtils.copyProperties(item,vxJobLikeDto);
            //获取岗位名称
            LambdaQueryWrapper<JobEntity> queryWrapper1=new LambdaQueryWrapper<>();
            queryWrapper1.eq(JobEntity::getId,item.getId());
            JobEntity jobById = jobMapper.selectOne(queryWrapper1);
            vxJobLikeDto.setJobName(jobById.getJobName());
            //获取薪水
            vxJobLikeDto.setSalary(jobById.getMinSalary()+"-"+jobById.getMaxSalary());
            //获取公司名称
            Company company=companyService.getCompanyDetail(item.getCompanyEncoding());
            vxJobLikeDto.setCompany(company);
            //获取员工人数
            //获取公司位置
            return vxJobLikeDto;
        }).collect(Collectors.toList());
        return jobDtos;
    }
}
