package com.yzdx.AiInterviewer.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yzdx.AiInterviewer.entity.JobEntity;
import com.yzdx.AiInterviewer.entity.JobPosting;
import com.yzdx.AiInterviewer.entity.dto.JobPostingDto;
import com.yzdx.AiInterviewer.mapper.JobMapper;
import com.yzdx.AiInterviewer.mapper.JobPostingMapper;
import com.yzdx.AiInterviewer.service.JobListService;
import com.yzdx.AiInterviewer.service.JobPostingService;
import com.yzdx.AiInterviewer.utiles.TimeUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.jws.Oneway;
import java.text.ParseException;
import java.text.SimpleDateFormat;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class JobPostingServiceImpl extends ServiceImpl<JobPostingMapper, JobPosting> implements JobPostingService {

    @Autowired
    private JobPostingMapper jobPostingMapper;

    @Autowired
    private JobListService jobListService;

    @Autowired
    private JobMapper jobMapper;

    @Override
    public Integer addJobPosting(String name, String startTime, String endTime, String checkList, Integer userId, String encoding) {

        LambdaQueryWrapper<JobPosting> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.eq(JobPosting::getRecruitmentName,name).eq(JobPosting::getCompanyEncoding,encoding);

        JobPosting findJobPosting = jobPostingMapper.selectOne(queryWrapper);

        if(findJobPosting!=null){

            return -2;

        }

        JobPosting newJobPosting=new JobPosting();

        newJobPosting.setRecruitmentName(name);

        newJobPosting.setStartTime(startTime);

        newJobPosting.setEndTime(endTime);

        newJobPosting.setJobId(checkList);

        newJobPosting.setCreateUser(userId);

        newJobPosting.setCreateTime(TimeUtil.getTime());

        newJobPosting.setUpdateUser(userId);

        newJobPosting.setUpdateTime(TimeUtil.getTime());

        newJobPosting.setCompanyEncoding(encoding);

        Integer rows = jobPostingMapper.insert(newJobPosting);

        return rows;
    }

    @Override
    public Integer updateJobPosting(Integer id, String name, String startTime, String endTime,
                                    String checkList, Integer userId,Integer status) {
        LambdaQueryWrapper<JobPosting> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.eq(JobPosting::getId,id);

        JobPosting findJobPosting = jobPostingMapper.selectOne(queryWrapper);

        findJobPosting.setRecruitmentName(name);

        findJobPosting.setStartTime(startTime);

        findJobPosting.setEndTime(endTime);

        findJobPosting.setJobId(checkList);

        findJobPosting.setUpdateUser(userId);

        findJobPosting.setUpdateTime(TimeUtil.getTime());

        findJobPosting.setStatus(status);

        Integer rows = jobPostingMapper.updateById(findJobPosting);

        return rows;
    }

    @Override
    public Integer deleteJobPosting(Integer id) {
        LambdaQueryWrapper<JobPosting> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.eq(JobPosting::getId,id);

        Integer rows = jobPostingMapper.delete(queryWrapper);

        return rows;
    }

    @Override
    public List<JobPostingDto> getJobPostingList(String encoding) {
        LambdaQueryWrapper<JobPosting> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.eq(JobPosting::getCompanyEncoding,encoding);

        List<JobPosting> jobPostings = jobPostingMapper.selectList(queryWrapper);

        List<JobPostingDto> JobPostingDtoList=jobPostings.stream().map(item->{
            JobPostingDto jobPostingDto=new JobPostingDto();

            BeanUtils.copyProperties(item,jobPostingDto);

            JSONArray jsonArray= (JSONArray) JSON.parse( item.getJobId());

            String JobName="";

            List<JobEntity> checkedList=new ArrayList<>();

            for (int i = 0; i < jsonArray.size(); i++) {

                JobEntity jobById = jobListService.getJobById((Integer) jsonArray.get(i));

                if(i==jsonArray.size()-1){
                    JobName=JobName+jobById.getJobName();
                    checkedList.add(jobById);
                    break;
                }
                JobName=jobById.getJobName()+",";
                checkedList.add(jobById);
            }

            jobPostingDto.setJobName(JobName);
            jobPostingDto.setCheckedList(checkedList);
            // 定义输入字符串的格式
            SimpleDateFormat inputStartTimeFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");
            // 定义输出字符串的格式
            SimpleDateFormat outputStartTimeFormat = new SimpleDateFormat("yyyy-MM-dd");
            String startTime ="";
            try {
                // 将输入字符串解析为日期对象
                Date date = inputStartTimeFormat.parse(item.getStartTime());
                // 将日期对象格式化为输出字符串
                startTime = outputStartTimeFormat.format(date);

            } catch (ParseException e) {
                e.printStackTrace();
            }

            // 定义输入字符串的格式
            SimpleDateFormat inputEndTimeFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");
            // 定义输出字符串的格式
            SimpleDateFormat outputEndTimeFormat = new SimpleDateFormat("yyyy-MM-dd");

            String endTime ="";
            try {
                // 将输入字符串解析为日期对象
                Date date = inputEndTimeFormat.parse(item.getStartTime());
                // 将日期对象格式化为输出字符串
                endTime = outputEndTimeFormat.format(date);

                // 输出结果
            } catch (ParseException e) {
                e.printStackTrace();
            }

            String Time=startTime+"至"+endTime;

            jobPostingDto.setTime(Time);



            return jobPostingDto;
        }).collect(Collectors.toList());

        return JobPostingDtoList;
    }

    @Override
    public List<Map<String, Object>> getPostingInfoList(String encoding) {
        List<Map<String, Object>> result=new ArrayList<>();

        LambdaQueryWrapper<JobPosting> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.eq(JobPosting::getCompanyEncoding,encoding);

        List<JobPosting> jobPostings = jobPostingMapper.selectList(queryWrapper);

        jobPostings.stream().map(item->{
            Map<String,Object> postInfo=new HashMap<>();

            JobPosting jobPosting = jobPostingMapper.selectById(item.getId());

            postInfo.put("postId",item.getId());

            postInfo.put("postName",item.getRecruitmentName());

            JSONArray jobListJSON=JSON.parseArray(jobPosting.getJobId());

            List<Map<String,Object>> jobList=new ArrayList<>();

            for (int i = 0; i <jobListJSON.size() ; i++) {

               Integer jobId= (Integer) jobListJSON.get(i);

                JobEntity jobEntity = jobMapper.selectById(jobId);

                Map<String, Object> jobItem=new HashMap<>();

                jobItem.put("jobId",jobEntity.getId());

                jobItem.put("jobName",jobEntity.getJobName());

                jobList.add(jobItem);

            }

            postInfo.put("jobList",jobList);

            result.add(postInfo);


        return null;
        }).collect(Collectors.toList());

        return result;

    }
}
