package com.yzdx.AiInterviewer.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yzdx.AiInterviewer.entity.QuestionBank;
import com.yzdx.AiInterviewer.mapper.QuestionBankMapper;
import com.yzdx.AiInterviewer.service.QuestionBankService;
import com.yzdx.AiInterviewer.utiles.TimeUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class QuestionBankServiceImpl extends ServiceImpl<QuestionBankMapper, QuestionBank> implements QuestionBankService {

    @Autowired
    private QuestionBankMapper questionBankMapper;


    @Override
    public List<QuestionBank> getTypeList(String encoding) {
        //若公司编码为空，则返回空
        if(encoding==null){
            return null;
        }
        LambdaQueryWrapper<QuestionBank> queryWrapper =new LambdaQueryWrapper<>();

        queryWrapper.eq(QuestionBank::getCompanyEncoding,encoding);

        List<QuestionBank> questionBanks= questionBankMapper.selectList(queryWrapper);

        return questionBanks;
    }

    @Override
    public List<QuestionBank> searchTypeList(String name, String type,String encoding) {

            if (!name.equals("")) {
                if (type.equals("")) {
                    LambdaQueryWrapper<QuestionBank> queryWrapper=new LambdaQueryWrapper<>();

                    queryWrapper.like(QuestionBank::getTypeName,name).eq(QuestionBank::getCompanyEncoding,encoding);

                    List<QuestionBank> questionBanks = questionBankMapper.selectList(queryWrapper);

                    return questionBanks;
                }else{
                    LambdaQueryWrapper<QuestionBank> queryWrapper=new LambdaQueryWrapper<>();

                    queryWrapper.like(QuestionBank::getTypeName,name).eq(QuestionBank::getCompanyEncoding,encoding).eq(QuestionBank::getType,type);

                    List<QuestionBank> questionBanks = questionBankMapper.selectList(queryWrapper);

                    return questionBanks;

                }

            }else {
                if(!type.equals("")){

                    LambdaQueryWrapper<QuestionBank> queryWrapper=new LambdaQueryWrapper<>();

                    queryWrapper.eq(QuestionBank::getType,type).eq(QuestionBank::getCompanyEncoding,encoding);

                    List<QuestionBank> questionBanks = questionBankMapper.selectList(queryWrapper);

                    return questionBanks;
                }else {
                    return null;
                }

            }


    }

    @Override
    public List<QuestionBank> getTypeListByType(String encoding, Integer type) {
        if(encoding==null||type==null){
            return null;
        }
        LambdaQueryWrapper<QuestionBank> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.eq(QuestionBank::getCompanyEncoding,encoding).eq(QuestionBank::getType,type);

        List<QuestionBank> selectList = questionBankMapper.selectList(queryWrapper);
        return selectList;
    }

    @Override
    public Integer addTypeName(String typeName, String encoding, String description,Integer type,Integer userId) {

        //判断题库名是否存在
        LambdaQueryWrapper<QuestionBank> queryWrapper =new LambdaQueryWrapper<>();
        queryWrapper.eq(QuestionBank::getTypeName,typeName).eq(QuestionBank::getCompanyEncoding,encoding);
        QuestionBank findQuestionBank=questionBankMapper.selectOne(queryWrapper);
        if(findQuestionBank!=null){
            //存在返回-2
            return -2;
        }

        QuestionBank insert=new QuestionBank();
        insert.setTypeName(typeName);
        insert.setCompanyEncoding(encoding);
        insert.setCreateUser(userId);
        insert.setUpdateUser(userId);
        insert.setDescription(description);
        insert.setType(type);
        insert.setCreateTime(TimeUtil.getTime());
        insert.setUpdateTime(TimeUtil.getTime());

        return questionBankMapper.insert(insert);

    }

    @Override
    public Integer deleteType(Integer typeId) {

        LambdaQueryWrapper<QuestionBank> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.eq(QuestionBank::getId,typeId);

      Integer row=  questionBankMapper.deleteById(typeId);

      return row;
    }

    @Override
    public Integer changeType(Integer typeId,String description,Integer type,String typeName,Integer userId) {
        //修改的题库名是否重名
        LambdaQueryWrapper<QuestionBank> queryWrapper =new LambdaQueryWrapper<>();

        queryWrapper.eq(QuestionBank::getTypeName,typeName);

        QuestionBank findBank=questionBankMapper.selectOne(queryWrapper);

        if(findBank!=null&& findBank.getId()!=typeId){
//            若重名，返回-2表示重名
            return -2;
        }
        //不重名，更新到数据库中
        //更新字段名：typeName,updateUser,updateTime
        LambdaQueryWrapper<QuestionBank> updateBankQueryWrapper=new LambdaQueryWrapper<>();

        updateBankQueryWrapper.eq(QuestionBank::getId,typeId);

        QuestionBank updateQuestionBank=questionBankMapper.selectOne(updateBankQueryWrapper);

        updateQuestionBank.setUpdateUser(userId);
       if(!description.equals("")){
           updateQuestionBank.setDescription(description);
       }
        updateQuestionBank.setUpdateTime(TimeUtil.getTime());

        updateQuestionBank.setTypeName(typeName);

        updateQuestionBank.setType(type);

       return  questionBankMapper.updateById(updateQuestionBank);
    }

    @Override
    public QuestionBank getBankById(Integer id) {
        LambdaQueryWrapper<QuestionBank> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.eq(QuestionBank::getId,id);

        return  questionBankMapper.selectOne(queryWrapper);
    }

}
