package com.yzdx.AiInterviewer.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yzdx.AiInterviewer.entity.JobExpectation;
import com.yzdx.AiInterviewer.entity.Resume;
import com.yzdx.AiInterviewer.entity.User;
import com.yzdx.AiInterviewer.mapper.JobExpectationMapper;
import com.yzdx.AiInterviewer.mapper.ResumeMapper;
import com.yzdx.AiInterviewer.mapper.UserMapper;
import com.yzdx.AiInterviewer.service.ResumeService;
import com.yzdx.AiInterviewer.utiles.TimeUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.Period;
import java.util.*;

@Service
public class ResumeServiceImpl extends ServiceImpl<ResumeMapper, Resume> implements ResumeService {

    @Autowired
    private ResumeMapper resumeMapper;

    @Autowired
    private UserMapper userMapper;
//    @Autowired
//    private JobExpectationMapper jobExpectationMapper;
    @Override
    public Map<String,Object> writeResume( String name, String avatar,String sex, String status, String graduateTime, String birthday,
                                String phone, String email, String vx, String personalAdvantage, String other,String education,
                                String jobExpectation, String workExperience, String projectExperience,
                                Integer userId) {

        Map<String,Object> result=new HashMap<>();

        LambdaQueryWrapper<Resume> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.eq(Resume::getUserId,userId);

        Resume resume1 = resumeMapper.selectOne(queryWrapper);

        if(resume1!=null){
            return null;
        }

        if(avatar!=null||email!=null){

            LambdaQueryWrapper<User> queryWrapper1=new LambdaQueryWrapper<User>();

            queryWrapper1.eq(User::getId,userId);

            User findUser = userMapper.selectOne(queryWrapper1);

            findUser.setUsername(name);

            findUser.setAvatar(avatar);

            findUser.setEmail(email);

            findUser.setSex((sex.equals("0")?"男":"女"));

            // 获取当前日期
            LocalDate currentDate = LocalDate.now();

            // 解析生日日期
            LocalDate birthDate = LocalDate.parse(birthday + "-01");

            // 计算年龄
            Period age = Period.between(birthDate, currentDate);


            findUser.setAge(String.valueOf(age.getYears()));


            Integer rows = userMapper.updateById(findUser);

            if(rows!=1){
                return null;
            }
        }

        Resume resume=new Resume();

        resume.setName(name);

        resume.setAvatar(avatar);

        resume.setSex(sex);

        resume.setStatus(status);

        resume.setGraduateTime(graduateTime);

        // 获取当前日期
        LocalDate currentDate = LocalDate.now();

        // 解析生日日期
        LocalDate birthDate = LocalDate.parse(birthday + "-01");

        // 计算年龄
        Period age = Period.between(birthDate, currentDate);

        resume.setAge(age.getYears());

        resume.setBirthday(birthday);

        resume.setPhone(phone);

        resume.setEmail(email);

        resume.setVx(vx);

        resume.setPersonalAdvantage(personalAdvantage);

        resume.setCreateTime(TimeUtil.getTime());

        resume.setUpdateTime(TimeUtil.getTime());

        resume.setOther(other);

        resume.setUpdateUser(userId);

        resume.setCreateUser(userId);

        resume.setUserId(userId);

        resume.setJobExpectation(jobExpectation);

        resume.setWorkExperience(workExperience);

        resume.setProjectExperience(projectExperience);

        resume.setEducationBackground(education);

        //解析在校经历，计算出学历

        JSONArray educationJSONArray= JSON.parseArray(education);

        long difference =0;

        String eduBack="";

        String school="";

        String major="";
        for (int i = 0; i < educationJSONArray.size(); i++) {

            Map<String,Object> educationMap= (Map<String,Object>)educationJSONArray.get(i);

            String lastEndTime= (String)educationMap.get("endTime");

            // 将字符串转换为日期对象
            Date date = new Date(Integer.valueOf(lastEndTime) - 1900, 0, 1); // 注意：年份需减去1900

            // 获取日期对象的时间戳（毫秒数）

            long timestamp = date.getTime();

            // 获取当前时间的时间戳
            long currentTimestamp = System.currentTimeMillis();

            // 计算时间戳差值
            long differenceTime = currentTimestamp - timestamp;

            //初始化值
            if(i==0){
                difference=differenceTime;
                eduBack=(String)educationMap.get("eduDegree");
                school=(String)educationMap.get("school");
                major=(String)educationMap.get("major");

                continue;
            }
            //判断最近的毕业年份
            if(differenceTime<difference){
                difference=differenceTime;
                eduBack=(String)educationMap.get("eduDegree");
                school=(String)educationMap.get("school");
                major=(String)educationMap.get("major");

                continue;
            }

        }

        resume.setEducation(eduBack);

        Integer rows= resumeMapper.insert(resume);

        if(rows!=1){
            return null;
        }
        int experience=0;
        //计算工作时间
        JSONArray jobJSONArray=JSON.parseArray(workExperience);

        if(jobJSONArray.size()==0) {
            //放入工作时间
            result.put("experience", experience);
        }else{
            for (int i = 0; i < jobJSONArray.size(); i++) {

                Map<String,Object> jobJSON=(Map<String, Object>) jobJSONArray.get(i);

                // 将字符串日期转换为LocalDate对象
                LocalDate startDate = LocalDate.parse(jobJSON.get("startTime")+ "-01");
                LocalDate endDate = LocalDate.parse(jobJSON.get("endTime") + "-01");

                // 计算年份差异
                Period period = Period.between(startDate, endDate);
                int yearDifference = period.getYears();
                experience+=yearDifference;
            }
            //放入工作时间
            result.put("experience", experience);
        }
        result.put("userName",name);
        result.put("userImgUrl",avatar);
        result.put("eduBack",eduBack);
        result.put("gender",sex);
        result.put("school",school);
        result.put("major",major);
        result.put("phone",phone);


        return result;
    }

//    @Override
//    public Integer addJobExpectation(Integer expectationType, String position, String location, String salary, Integer userId) {
//        JobExpectation jobExpectation=new JobExpectation();
//        jobExpectation.setExpectationType(expectationType);
//        jobExpectation.setPosition(position);
//        jobExpectation.setSalary(salary);
//        jobExpectation.setCreateTime(TimeUtil.getTime());
//        jobExpectation.setUpdateTime(TimeUtil.getTime());
//        jobExpectation.setCreateUser(userId);
//        jobExpectation.setUpdateUser(userId);
//        Integer rows = jobExpectationMapper.insert(jobExpectation);
//        return rows;
//    }

//    @Override
//    public List<JobExpectation> getJobExpectation(Integer userId) {
//        LambdaQueryWrapper<JobExpectation> queryWrapper=new LambdaQueryWrapper<>();
//        queryWrapper.eq(JobExpectation::getUserId,userId);
//        List<JobExpectation> list = jobExpectationMapper.selectList(queryWrapper);
//        return list;
//    }

//    @Override
//    public Integer updateJobExpectation(Integer id,Integer expectationType, String position, String location, String salary, Integer userId) {
//        JobExpectation jobExpectation=new JobExpectation();
//        jobExpectation.setId(id);
//        jobExpectation.setExpectationType(expectationType);
//        jobExpectation.setPosition(position);
//        jobExpectation.setSalary(salary);
//        jobExpectation.setCreateTime(TimeUtil.getTime());
//        jobExpectation.setUpdateTime(TimeUtil.getTime());
//        jobExpectation.setCreateUser(userId);
//        jobExpectation.setUpdateUser(userId);
//        Integer rows = jobExpectationMapper.updateById(jobExpectation);
//        return rows;
//    }
//
//    @Override
//    public Integer deleteJobExpectation(Integer id) {
//        LambdaQueryWrapper<JobExpectation> queryWrapper=new LambdaQueryWrapper<>();
//        queryWrapper.eq(JobExpectation::getId,id);
//        if(jobExpectationMapper.selectOne(queryWrapper)==null){
//            return -2;
//        }
//        Integer rows = jobExpectationMapper.delete(queryWrapper);
//        return rows;
//    }

    @Override
    public Map<String,Object> updateResume(Integer id, String name, String avatar,String sex, String status,
                                String graduateTime, String birthday, String phone, String email, String vx,
                                String personalAdvantage, String other,String education,String jobExpectation, String workExperience,
                                String projectExperience, Integer userId) {

        Map<String,Object> result=new HashMap<>();
        if(avatar!=null&&name!=null&&email!=null&&phone!=null){

            LambdaQueryWrapper<User> queryWrapper1=new LambdaQueryWrapper<User>();

            queryWrapper1.eq(User::getId,userId);

            User findUser = userMapper.selectOne(queryWrapper1);

            findUser.setUsername(name);

            findUser.setAvatar(avatar);

            findUser.setEmail(email);

            Integer rows = userMapper.updateById(findUser);

        }

        Resume resume=new Resume();
        resume.setId(id);
        resume.setName(name);
        resume.setAvatar(avatar);
        resume.setSex(sex);
        resume.setStatus(status);
        resume.setGraduateTime(graduateTime);
        resume.setBirthday(birthday);
        resume.setPhone(phone);
        resume.setEmail(email);
        resume.setVx(vx);
        resume.setPersonalAdvantage(personalAdvantage);
        resume.setUpdateTime(TimeUtil.getTime());
        resume.setOther(other);
        resume.setUpdateUser(userId);
        resume.setJobExpectation(jobExpectation);
        resume.setWorkExperience(workExperience);
        resume.setProjectExperience(projectExperience);

        //解析在校经历，计算出学历

        JSONArray educationJSONArray= JSON.parseArray(education);

        long difference =0;

        String eduBack="";

        String school="";

        String major="";
        for (int i = 0; i < educationJSONArray.size(); i++) {

            Map<String,Object> educationMap= (Map<String,Object>)educationJSONArray.get(i);

            String lastEndTime= (String)educationMap.get("endTime");

            // 将字符串转换为日期对象
            Date date = new Date(Integer.valueOf(lastEndTime) - 1900, 0, 1); // 注意：年份需减去1900

            // 获取日期对象的时间戳（毫秒数）

            long timestamp = date.getTime();

            // 获取当前时间的时间戳
            long currentTimestamp = System.currentTimeMillis();

            // 计算时间戳差值
            long differenceTime = currentTimestamp - timestamp;

            //初始化值
            if(i==0){
                difference=differenceTime;
                eduBack=(String)educationMap.get("eduDegree");
                school=(String)educationMap.get("school");
                major=(String)educationMap.get("major");

                continue;
            }
            //判断最近的毕业年份
            if(differenceTime<difference){
                difference=differenceTime;
                eduBack=(String)educationMap.get("eduDegree");
                school=(String)educationMap.get("school");
                major=(String)educationMap.get("major");

                continue;
            }

        }

        resume.setEducation(eduBack);

        Integer rows= resumeMapper.updateById(resume);

        if(rows!=1){
            return null;
        }
        int experience=0;
        //计算工作时间
        JSONArray jobJSONArray=JSON.parseArray(workExperience);

        if(jobJSONArray.size()==0) {
            //放入工作时间
            result.put("experience", experience);
        }else{
            for (int i = 0; i < jobJSONArray.size(); i++) {

                Map<String,Object> jobJSON=(Map<String, Object>) jobJSONArray.get(i);

                // 将字符串日期转换为LocalDate对象
                LocalDate startDate = LocalDate.parse(jobJSON.get("startTime")+ "-01");
                LocalDate endDate = LocalDate.parse(jobJSON.get("endTime") + "-01");

                // 计算年份差异
                Period period = Period.between(startDate, endDate);
                int yearDifference = period.getYears();
                experience+=yearDifference;
            }
            //放入工作时间
            result.put("experience", experience);
        }
        result.put("userName",name);
        result.put("userImgUrl",avatar);
        result.put("eduBack",eduBack);
        result.put("gender",sex);
        result.put("school",school);
        result.put("major",major);
        result.put("phone",phone);

        result.put("resumeId",resume.getId());
        return result;
    }

    @Override
    public  Map<String,Object> getResume(Integer userId) {


        LambdaQueryWrapper<Resume> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.eq(Resume::getUserId,userId);

        Resume resume = resumeMapper.selectOne(queryWrapper);

        if(resume==null){
            return null;
        }

        //解析在校经历，计算出学历

        JSONArray educationJSONArray= JSON.parseArray(resume.getEducationBackground());

        long difference =0;

        String eduBack="";

        String school="";

        String major="";
        for (int i = 0; i < educationJSONArray.size(); i++) {

            Map<String,Object> educationMap= (Map<String,Object>)educationJSONArray.get(i);

            String lastEndTime= (String)educationMap.get("endTime");

            // 将字符串转换为日期对象
            Date date = new Date(Integer.valueOf(lastEndTime) - 1900, 0, 1); // 注意：年份需减去1900

            // 获取日期对象的时间戳（毫秒数）

            long timestamp = date.getTime();

            // 获取当前时间的时间戳
            long currentTimestamp = System.currentTimeMillis();

            // 计算时间戳差值
            long differenceTime = currentTimestamp - timestamp;

            //初始化值
            if(i==0){
                difference=differenceTime;
                eduBack=(String)educationMap.get("eduDegree");
                school=(String)educationMap.get("school");
                major=(String)educationMap.get("major");

                continue;
            }
            //判断最近的毕业年份
            if(differenceTime<difference){
                difference=differenceTime;
                eduBack=(String)educationMap.get("eduDegree");
                school=(String)educationMap.get("school");
                major=(String)educationMap.get("major");

                continue;
            }

        }

        resume.setEducation(eduBack);

        Integer rows= resumeMapper.updateById(resume);

        Map<String,Object> result=new HashMap<>();

        if(rows!=1){
            return null;
        }
        int experience=0;
        //计算工作时间
        JSONArray jobJSONArray=JSON.parseArray(resume.getWorkExperience());

        if(jobJSONArray.size()==0) {
            //放入工作时间
            result.put("experience", experience);
        }else{
            for (int i = 0; i < jobJSONArray.size(); i++) {

                Map<String,Object> jobJSON=(Map<String, Object>) jobJSONArray.get(i);

                // 将字符串日期转换为LocalDate对象
                LocalDate startDate = LocalDate.parse(jobJSON.get("startTime")+ "-01");
                LocalDate endDate = LocalDate.parse(jobJSON.get("endTime") + "-01");

                // 计算年份差异
                Period period = Period.between(startDate, endDate);
                int yearDifference = period.getYears();
                experience+=yearDifference;
            }
            //放入工作时间
            result.put("experience", experience);
        }
        result.put("userName",resume.getName());
        result.put("userImgUrl",resume.getAvatar());
        result.put("eduBack",eduBack);
        result.put("gender",resume.getSex());
        result.put("school",school);
        result.put("major",major);
        result.put("phone",resume.getPhone());
        result.put("resume",resume);

        return result;

    }
}
