package com.yzdx.AiInterviewer.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yzdx.AiInterviewer.comment.R;
import com.yzdx.AiInterviewer.entity.BlackToken;
import com.yzdx.AiInterviewer.entity.Company;
import com.yzdx.AiInterviewer.entity.User;
import com.yzdx.AiInterviewer.mapper.BlackTokenMapper;
import com.yzdx.AiInterviewer.mapper.CompanyMapper;
import com.yzdx.AiInterviewer.mapper.UserMapper;
import com.yzdx.AiInterviewer.service.UserService;
import com.yzdx.AiInterviewer.utiles.JWT;
import com.yzdx.AiInterviewer.utiles.MD5Util;
import com.yzdx.AiInterviewer.utiles.TimeUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements UserService {

    @Autowired
    private UserMapper userMapper;
    @Autowired
    private BlackTokenMapper blackTokenMapper;
    @Autowired
    private CompanyMapper companyMapper;


    @Override
    public R adminLogin(String phone, String encoding, String password) {
        //判断账号和密码是否正确
        LambdaQueryWrapper<User> userLambdaQueryWrapper=new LambdaQueryWrapper<>();

        userLambdaQueryWrapper.eq(User::getPhone,phone).eq(User::getCompanyEncoding,encoding);

        User selectUser = userMapper.selectOne(userLambdaQueryWrapper);

        if(selectUser==null){
            return R.error("账号或密码有误，请检查输入！");
        }

        String salt=selectUser.getSalt();

        String selectPassword=selectUser.getPassword();

       password = MD5Util.GetMD5Password(password, salt);

       if(!password.equals(selectPassword)){

           return R.error("账号或密码有误，请检查输入！");
       }

        if(selectUser.getRole().equals("4")){
            return R.error("账号权限不足，请联系管理员");
        }
        //均正确，返回token密钥
        String token=JWT.getJWToken(selectUser.getId());

        selectUser.setPassword(null);
        selectUser.setSalt(null);

        LambdaQueryWrapper<Company> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.eq(Company::getEncoding,encoding);

        Company company = companyMapper.selectOne(queryWrapper);

        Map<String,Object> data=new HashMap<>();

        data.put("token",token);
        data.put("encoding",encoding);
        data.put("companyName",company.getCompanyName());
        data.put("userInfo",selectUser);

        return R.success(data);
    }
    @Override
    public User getUserById(Integer userid) {

        LambdaQueryWrapper<User> queryWrapper =new LambdaQueryWrapper<>();
        queryWrapper.eq(User::getId,userid);
        return userMapper.selectOne(queryWrapper);
    }

    @Override
    public Integer adminLogout(String token) {
        //判断token的正确性
        boolean verify = JWT.verify(token);
        if(!verify){
            //若不正确返回0
            return 0;
        }
        //正确的话,将token存入黑名单里面

        Integer userId = JWT.getTokenId(token);

        BlackToken blackToken=new BlackToken();

        blackToken.setUserId(userId);

        blackToken.setValue(token);

        blackToken.setCreateTime(TimeUtil.getPreciseTime());

        Integer rows=blackTokenMapper.insert(blackToken);
        if(rows==0){
            return 0;
        }

        return 1;
    }

    @Override
    public List<User> getAdminList(String encoding) {
        //根据公司编码查询管理员
        LambdaQueryWrapper<User> userLambdaQueryWrapper=new LambdaQueryWrapper<>();

        userLambdaQueryWrapper.eq(User::getCompanyEncoding,encoding);

        List<User> userList = userMapper.selectList(userLambdaQueryWrapper);

        List<User> filterUserList = userList.stream().map((item) -> {
            User user=new User();

            BeanUtils.copyProperties(item,user);

           user.setPassword(null);
           user.setSalt(null);
            return user;
        }).collect(Collectors.toList());
        return filterUserList;
    }

    @Override
    public List<User> searchAdmin(String name, String encoding) {
        LambdaQueryWrapper<User> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.like(User::getUsername,name).eq(User::getCompanyEncoding,encoding);

        List<User> userList = userMapper.selectList(queryWrapper);

        List<User> filterUserList = userList.stream().map((item) -> {
            User user=new User();

            BeanUtils.copyProperties(item,user);

            user.setPassword(null);
            user.setSalt(null);
            return user;
        }).collect(Collectors.toList());
        return filterUserList;
    }
//系统管理添加HR
    @Override
    public Integer addAdmin(String encoding, Integer userId, String username, String phone,String role) {

       //查询操作人的权限(添加管理员的权限应该是公司管理员)
        //公司管理员-》公司管理员, HR
        User user = userMapper.selectById(userId);

        String userRole =user.getRole();

        if(userRole.equals("3")){

            return -3;
        }
        LambdaQueryWrapper<User> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.eq(User::getPhone,phone);

        User findUser = userMapper.selectOne(queryWrapper);

        if(findUser!=null){
            //用户转hr
            if(findUser.getRole().equals("4")){
                findUser.setRole(role);
                findUser.setCompanyEncoding(encoding);
                return userMapper.updateById(findUser);
            }
            return -2;
        }
        User newUser=new User();
        newUser.setUsername(username);
        newUser.setPhone(phone);
        String salt=MD5Util.getSalt();
        newUser.setSalt(salt);
        String password=phone.substring(phone.length()-6);
        newUser.setPassword(MD5Util.GetMD5Password(password,salt));
        newUser.setRole(role);
        newUser.setCompanyEncoding(encoding);
        newUser.setCreateUser(userId);
        newUser.setCreateTime(TimeUtil.getTime());
        newUser.setUpdateUser(userId);
        newUser.setUpdateTime(TimeUtil.getTime());

        Integer rows = userMapper.insert(newUser);

        return rows;
    }

    @Override
    public Integer deleteAdmin(Integer deleteId, Integer userId) {
        //判断是否是删除本人
        if(deleteId==userId){
            return -3;
        }

        //判断有误越权
        User user = userMapper.selectById(userId);

        if(user.getRole().equals("3")||user.getRole().equals("4")){
            return -2;
        }
        Integer rows = userMapper.deleteById(deleteId);

        return rows;
    }

    @Override
    public User changeAvatar(Integer userId, String avatar) {
        LambdaQueryWrapper<User> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.eq(User::getId,userId);
        User findUser = userMapper.selectOne(queryWrapper);
        if(findUser==null){
            return null;
        }
        findUser.setAvatar(avatar);
        Integer rows = userMapper.updateById(findUser);
        if (rows==0){
            return null;
        }
        findUser.setPassword(null);
        findUser.setSalt(null);
        return findUser;
    }

    @Override
    public R resetPassword(Integer userId, String oldPassword, String password) {

        LambdaQueryWrapper<User> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.eq(User::getId,userId);

        User findUser = userMapper.selectOne(queryWrapper);

        if(findUser==null){
            return R.error("重置密码失败，请联系管理员！");
        }
        //判断旧密码是否正确

        String salt = findUser.getSalt();
        String findUserPassword = findUser.getPassword();
        oldPassword  = MD5Util.GetMD5Password(oldPassword, salt);
        if(!findUserPassword.equals(oldPassword)){
            return R.error("旧密码输入错误，请检查输入，或联系管理员修改密码！");
        }
        //密码正确
        password = MD5Util.GetMD5Password(password, salt);

        //更新密码
        findUser.setPassword(password);
        findUser.setUpdateUser(userId);
        findUser.setUpdateTime(TimeUtil.getTime());

        Integer row = userMapper.updateById(findUser);
        if(row==0){
            return R.error("重置密码失败，请联系管理员！");
        }
        return R.success("重置密码成功!");

    }

    @Override
    public User updateUserInfo(Integer userId, Integer updateId,String username, String age, String email, String sex) {
        //判断是否用户自己修改自己的数据
        if (userId==null||userId.equals("")) {
            //查找对象
            LambdaQueryWrapper<User> queryWrapper=new LambdaQueryWrapper<>();

            queryWrapper.eq(User::getId,updateId);

            User findUser = userMapper.selectOne(queryWrapper);

            findUser.setUsername(username);

            findUser.setAge(age);

            findUser.setEmail(email);

            findUser.setSex(sex);

            findUser.setUpdateUser(updateId);

            findUser.setUpdateTime(TimeUtil.getTime());

             userMapper.updateById(findUser);

            //对敏感数据进行处理
            findUser.setPassword("");

            findUser.setSalt("");

            return findUser;
        }else {

            return null;
        }

    }

    @Override
    public R vxLogin(String phone) {

        LambdaQueryWrapper<User> queryWrapper=new LambdaQueryWrapper<>();

        queryWrapper.eq(User::getPhone,phone);

        User user = userMapper.selectOne(queryWrapper);

        if(user==null){
            User createUser=new User();
            createUser.setPhone(phone);
            createUser.setCreateTime(TimeUtil.getTime());
            createUser.setUsername(MD5Util.getSalt());
            userMapper.insert(createUser);

            LambdaQueryWrapper<User> findNewQueryWrapper=new LambdaQueryWrapper<>();

            findNewQueryWrapper.eq(User::getPhone,phone);

            User findNew = userMapper.selectOne(queryWrapper);

            String jwToken = JWT.getJWToken(findNew.getId());

            Map<String,Object> result=new HashMap<>();

            result.put("token",jwToken);
            result.put("userInfo",findNew);
            return R.success(result);
        }else{
            String jwToken = JWT.getJWToken(user.getId());

            Map<String,Object> result=new HashMap<>();

            result.put("token",jwToken);
            result.put("userInfo",user);
            return R.success(result);
        }

    }

    @Override
    public Integer adminEditPassword(Integer userId, Integer updateUserId, String password) {
        //判断权限是否越界
        User user = userMapper.selectById(userId);
        User updateUser = userMapper.selectById(updateUserId);
        if(user.getRole().equals(updateUser.getRole())&&userId!=updateUserId){
            return -2;
        }
        if(user.getRole().equals("3")||user.getRole().equals("4")){

            return -3;
        }

        String salt = updateUser.getSalt();

        String newPassword = MD5Util.GetMD5Password(password, salt);

        updateUser.setPassword(newPassword);

        updateUser.setUpdateUser(userId);

        updateUser.setUpdateTime(TimeUtil.getTime());

        return userMapper.updateById(updateUser);

    }

}

